@@ -8,6 +8,7 @@
  *	Based on ff-csv code.
  */
 #include <gnumeric-config.h>
+#include <gnumeric-i18n.h>
 #include <gnumeric.h>
 
 #include <cell.h>
@@ -20,17 +21,19 @@
 #include <module-plugin-defs.h>
 
 #include <gsf/gsf-input-textline.h>
+#include <gsf/gsf-output.h>
+#include <gsf/gsf-utils.h>
 #include <string.h>
-#include <libgnome/gnome-i18n.h>
+#include <stdlib.h>
 
 GNUMERIC_MODULE_PLUGIN_INFO_DECL;
 
 #define N_INPUT_LINES_BETWEEN_UPDATES   50
 
-void dif_file_open (GnumFileOpener const *fo, IOContext *io_context,
+void dif_file_open (GnmFileOpener const *fo, IOContext *io_context,
                     WorkbookView *wbv, GsfInput *input);
-void dif_file_save (GnumFileSaver const *fs, IOContext *io_context,
-                    WorkbookView *wbv, const gchar *file_name);
+void dif_file_save (GnmFileSaver const *fs, IOContext *io_context,
+                    WorkbookView const *wbv, GsfOutput *output);
 
 typedef struct {
 	IOContext *io_context;
@@ -40,6 +43,8 @@ typedef struct {
 	gchar *line;
 
 	Sheet *sheet;
+
+	GIConv converter;
 } DifInputContext;
 
 
@@ -55,6 +60,7 @@ dif_input_context_new (IOContext *io_context, Workbook *wb, GsfInput *input)
 	ctxt->line_no        = 1;
 	ctxt->line           = NULL;
 	ctxt->sheet          = workbook_sheet_add (wb, NULL, FALSE);
+	ctxt->converter      = g_iconv_open ("UTF-8", "ISO-8859-1");
 
 	io_progress_message (io_context, _("Reading file..."));
 
@@ -66,13 +72,24 @@ dif_input_context_destroy (DifInputContext *ctxt)
 {
 	io_progress_unset (ctxt->io_context);
 	g_object_unref (G_OBJECT (ctxt->input)); ctxt->input = NULL;
+	gsf_iconv_close (ctxt->converter);
+	g_free (ctxt->line);
 	g_free (ctxt);
 }
 
 static gboolean
 dif_get_line (DifInputContext *ctxt)
 {
-	ctxt->line = gsf_input_textline_ascii_gets (ctxt->input);
+	char *raw;
+
+	raw = gsf_input_textline_ascii_gets (ctxt->input);
+	if (!raw)
+		return FALSE;
+
+	g_free (ctxt->line);
+	ctxt->line =
+		g_convert_with_iconv (raw, -1, ctxt->converter, NULL, NULL, NULL);
+
 	return ctxt->line != NULL;
 }
 
@@ -234,44 +251,35 @@ dif_parse_sheet (DifInputContext *ctxt)
 }
 
 void
-dif_file_open (GnumFileOpener const *fo, IOContext *io_context,
+dif_file_open (GnmFileOpener const *fo, IOContext *io_context,
                WorkbookView *wbv, GsfInput *input)
 {
-	DifInputContext *ctxt;
+	Workbook *wb = wb_view_workbook (wbv);
+	DifInputContext *ctxt = dif_input_context_new (io_context, wb, input);
 
-	ctxt = dif_input_context_new (io_context, wb_view_workbook (wbv), input);
+	workbook_set_saveinfo (wb, FILE_FL_MANUAL_REMEMBER,
+		gnm_file_saver_for_id ("Gnumeric_dif:dif"));
 	if (ctxt != NULL) {
 		dif_parse_sheet (ctxt);
-		if (gnumeric_io_error_occurred (io_context)) {
-			gnumeric_io_error_push (io_context, error_info_new_str (
-			_("Error while reading DIF file.")));
-		}
+		if (gnumeric_io_error_occurred (io_context))
+			gnumeric_io_error_push (io_context,
+				error_info_new_str (_("Error while reading DIF file.")));
 		dif_input_context_destroy (ctxt);
-	} else {
-		if (!gnumeric_io_error_occurred) {
-			gnumeric_io_error_unknown (io_context);
-		}
-	}
+	} else if (!gnumeric_io_error_occurred)
+		gnumeric_io_error_unknown (io_context);
 }
 
 /*
  * Write _current_ sheet of the workbook to a DIF format file
  */
 void
-dif_file_save (GnumFileSaver const *fs, IOContext *io_context,
-               WorkbookView *wbv, const gchar *file_name)
+dif_file_save (GnmFileSaver const *fs, IOContext *io_context,
+               WorkbookView const *wbv, GsfOutput *output)
 {
-	FILE *f;
-	ErrorInfo *open_error;
 	Sheet *sheet;
 	Range r;
 	gint row, col;
-
-	f = gnumeric_fopen_error_info (file_name, "w", &open_error);
-	if (f == NULL) {
-		gnumeric_io_error_info_set (io_context, open_error);
-		return;
-	}
+	gboolean res;
 
 	sheet = wb_view_cur_sheet (wbv);
 	if (sheet == NULL) {
@@ -282,34 +290,37 @@ dif_file_save (GnumFileSaver const *fs, IOContext *io_context,
 	r = sheet_get_extent (sheet, FALSE);
 
 	/* Write out the standard headers */
-	fputs ("TABLE\n" "0,1\n" "\"GNUMERIC\"\n", f);
-	fprintf (f, "VECTORS\n" "0,%d\n" "\"\"\n", r.end.row);
-	fprintf (f, "TUPLES\n" "0,%d\n" "\"\"\n", r.end.col);
-	fputs ("DATA\n0,0\n" "\"\"\n", f);
+	res  = gsf_output_puts (output, "TABLE\n" "0,1\n" "\"GNUMERIC\"\n");
+	if (res) res = gsf_output_printf (output,
+					  "VECTORS\n" "0,%d\n" "\"\"\n",
+					  r.end.row);
+	if (res) res = gsf_output_printf (output,
+					  "TUPLES\n" "0,%d\n" "\"\"\n",
+					  r.end.col);
+	if (res) res= gsf_output_puts (output, "DATA\n0,0\n" "\"\"\n");
 
 	/* Process all cells */
-	for (row = r.start.row; row <= r.end.row; row++) {
-		fputs ("-1,0\n" "BOT\n", f);
+	for (row = r.start.row; res && row <= r.end.row; row++) {
+		gsf_output_puts (output, "-1,0\n" "BOT\n");
 		for (col = r.start.col; col <= r.end.col; col++) {
 			Cell *cell;
 
 			cell = sheet_cell_get (sheet, col, row);
 			if (cell_is_blank (cell)) {
-				fputs("1,0\n" "\"\"\n", f);
+				gsf_output_puts(output, "1,0\n" "\"\"\n");
 			} else {
 				gchar *str;
 
 				str = cell_get_rendered_text (cell);
-				fprintf (f, "1.0\n" "\"%s\"\n", str);
+				res = gsf_output_printf (output,
+							 "1.0\n" "\"%s\"\n",
+							 str);
 				g_free (str);
 			}
 		}
 	}
-	fputs ("-1,0\n" "EOD\n", f);
+	gsf_output_puts (output, "-1,0\n" "EOD\n");
 
-	if (ferror (f)) {
+	if (!res)
 		gnumeric_io_error_string (io_context, _("Error while saving DIF file."));
-	}
-
-	fclose (f);
 }
