@@ -30,6 +30,7 @@ typedef enum {
     /* Integers & Enums */
 	MS_OBJ_ATTR_IS_INT_MASK = 0x1000,
 	MS_OBJ_ATTR_BLIP_ID,
+	MS_OBJ_ATTR_FONT_COLOR,
 	MS_OBJ_ATTR_FILL_COLOR,
 	MS_OBJ_ATTR_OUTLINE_COLOR,
 	MS_OBJ_ATTR_SCROLLBAR_VALUE,
@@ -97,7 +98,7 @@ struct _MSObj {
 	int id;
 
 	/* Type specific parameters */
-	GObject		*gnum_obj;
+	SheetObject	*gnum_obj;
 	int		 excel_type;
 	char const	*excel_type_name;
 
