@@ -4,8 +4,22 @@
  * Author:
  *  Miguel de Icaza (miguel@gnu.org)
  *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 #include <gnumeric-config.h>
+#include <gnumeric-i18n.h>
 #include <gnumeric.h>
 #include "dialogs.h"
 
@@ -18,10 +32,7 @@
 #include <gtk/gtk.h>
 #include <commands.h>
 
-#include <libgnome/gnome-i18n.h>
-
-#define GLADE_FILE "cell-comment.glade"
-#define COMMENT_DIALOG_KEY "zoom-dialog"
+#define COMMENT_DIALOG_KEY "cell-comment-dialog"
 
 typedef struct {
 	WorkbookControlGUI *wbcg;
@@ -55,20 +66,21 @@ dialog_cell_comment_destroy (GtkObject *w, CommentState *state)
 }
 
 static void
-cb_cell_comment_cancel_clicked (GtkWidget *button, CommentState *state)
+cb_cell_comment_cancel_clicked (G_GNUC_UNUSED GtkWidget *button,
+				CommentState *state)
 {
 	gtk_widget_destroy (state->dialog);
-	return;
 }
 
 static void
-cb_cell_comment_ok_clicked (GtkWidget *button, CommentState *state)
+cb_cell_comment_ok_clicked (G_GNUC_UNUSED GtkWidget *button,
+			    CommentState *state)
 {
 	GtkTextIter start;
 	GtkTextIter end;
 	char *text;
 
-	gtk_text_buffer_get_bounds  (state->text, &start, &end);
+	gtk_text_buffer_get_bounds (state->text, &start, &end);
 	text = gtk_text_buffer_get_text (state->text, &start, &end, TRUE);
 
 	if (!cmd_set_comment (WORKBOOK_CONTROL (state->wbcg), state->sheet, state->pos, text))
@@ -80,9 +92,10 @@ cb_cell_comment_ok_clicked (GtkWidget *button, CommentState *state)
 void
 dialog_cell_comment (WorkbookControlGUI *wbcg, Sheet *sheet, CellPos const *pos)
 {
-	CommentState *state;
-	GtkWidget *textview;
-	CellComment   *comment;
+	CommentState	*state;
+	GtkWidget	*textview;
+	CellComment	*comment;
+	GladeXML	*gui;
 
 	g_return_if_fail (wbcg != NULL);
 	g_return_if_fail (sheet != NULL);
@@ -90,14 +103,16 @@ dialog_cell_comment (WorkbookControlGUI *wbcg, Sheet *sheet, CellPos const *pos)
 
 	if (gnumeric_dialog_raise_if_exists (wbcg, COMMENT_DIALOG_KEY))
 		return;
+	gui = gnm_glade_xml_new (COMMAND_CONTEXT (wbcg),
+		"cell-comment.glade", NULL, NULL);
+	if (gui == NULL)
+		return;
 
 	state = g_new (CommentState, 1);
 	state->wbcg  = wbcg;
-	state->sheet  = sheet;
-	state->pos  = pos;
-
-	state->gui = gnumeric_glade_xml_new (wbcg, GLADE_FILE);
-	g_return_if_fail (state->gui != NULL);
+	state->sheet = sheet;
+	state->pos   = pos;
+	state->gui   = gui;
 
 	state->dialog = glade_xml_get_widget (state->gui, "comment_dialog");
 	g_return_if_fail (state->dialog != NULL);
@@ -107,9 +122,14 @@ dialog_cell_comment (WorkbookControlGUI *wbcg, Sheet *sheet, CellPos const *pos)
 	state->text = gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview));
 
 	comment = cell_has_comment_pos (sheet, pos);
-	if (comment)
-		gtk_text_buffer_set_text (state->text, cell_comment_text_get (comment), 
+	if (comment) {
+		GtkTextIter start;
+
+		gtk_text_buffer_set_text (state->text, cell_comment_text_get (comment),
 					  -1);
+		gtk_text_buffer_get_start_iter (state->text, &start);
+		gtk_text_buffer_place_cursor (state->text, &start);
+	}
 
 	state->ok_button = glade_xml_get_widget (state->gui, "ok_button");
 	g_signal_connect (G_OBJECT (state->ok_button),
@@ -129,9 +149,10 @@ dialog_cell_comment (WorkbookControlGUI *wbcg, Sheet *sheet, CellPos const *pos)
 		"destroy",
 		G_CALLBACK (dialog_cell_comment_destroy), state);
 
+	gtk_widget_grab_focus (textview);
+
 	wbcg_edit_attach_guru (state->wbcg, state->dialog);
 	gnumeric_keyed_dialog (state->wbcg, GTK_WINDOW (state->dialog),
 			       COMMENT_DIALOG_KEY);
 	gtk_widget_show_all (state->dialog);
-	gtk_widget_grab_focus (textview);
 }
