@@ -19,6 +19,8 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#undef GTK_DISABLE_DEPRECATED
+#warning "This file uses GTK_DISABLE_DEPRECATED"
 #include <gnumeric-config.h>
 #include <gnumeric-i18n.h>
 #include <gnumeric.h>
@@ -27,8 +29,7 @@
 
 #include <command-context.h>
 #include <gdk/gdkkeysyms.h>
-
-#define GLADE_FILE "dialog-stf-export.glade"
+#include <workbook.h>
 
 /**
  * stf_export_dialog_druid_page_cancel
@@ -41,7 +42,8 @@
  * returns : TRUE if the user actually wants to cancel, FALSE otherwise.
  **/
 static gboolean
-stf_export_dialog_druid_page_cancel (GnomeDruidPage *page, GnomeDruid *druid,
+stf_export_dialog_druid_page_cancel (G_GNUC_UNUSED GnomeDruidPage *page,
+				     G_GNUC_UNUSED GnomeDruid *druid,
 				     StfE_DruidData_t *druid_data)
 {
 	return !gnumeric_dialog_question_yes_no (druid_data->wbcg,
@@ -353,24 +355,13 @@ stf_export_dialog (WorkbookControlGUI *wbcg, Workbook *wb)
 	GladeXML *gui;
 	StfE_Result_t *dialogresult;
 	StfE_DruidData_t druid_data;
-	char* message;
 
 	g_return_val_if_fail (wb != NULL, NULL);
 
-	gui = gnumeric_glade_xml_new (NULL, GLADE_FILE);
-	if (!gui) {
-
-		message = g_strdup_printf (_("Missing %s file"), GLADE_FILE);
-
-		if (wbcg)
-			gnumeric_error_read (COMMAND_CONTEXT (wbcg), message);
-		else
-			g_warning (message);
-
-		g_free (message);
-
+	gui = gnm_glade_xml_new (COMMAND_CONTEXT (wbcg),
+		"dialog-stf-export.glade", NULL, NULL);
+	if (gui == NULL)
 		return NULL;
-	}
 
 	druid_data.wbcg             = wbcg;
 	druid_data.canceled         = FALSE;
@@ -382,7 +373,13 @@ stf_export_dialog (WorkbookControlGUI *wbcg, Workbook *wb)
 	stf_export_dialog_editables_enter (&druid_data);
 	gtk_widget_grab_default (druid_data.druid->next);
 
-	gnumeric_set_transient (wbcg, druid_data.window);
+	gnumeric_set_transient (wbcg_toplevel (wbcg), druid_data.window);
+	if (workbook_sheet_count (wb) == 1) {
+		stf_export_dialog_druid_page_next (druid_data.sheet_page,
+						   druid_data.druid,
+						   &druid_data);
+		gnome_druid_set_buttons_sensitive (druid_data.druid,
+						   FALSE, TRUE, TRUE, TRUE);	}
 	gtk_widget_show (GTK_WIDGET (druid_data.window));
 
 	gtk_main ();
