@@ -24,14 +24,11 @@
 #include <gnumeric-config.h>
 #include <gnumeric-i18n.h>
 #include "gnumeric.h"
-#include "numbers.h"
 #include "format.h"
 
 #include "style-color.h"
-#include "expr.h"
 #include "dates.h"
 #include "value.h"
-#include "parse-util.h"
 #include "datetime.h"
 #include "mathfunc.h"
 #include "str.h"
@@ -49,6 +46,13 @@
 
 /***************************************************************************/
 
+static StyleFormat *default_percentage_fmt;
+static StyleFormat *default_money_fmt;
+static StyleFormat *default_date_fmt;
+static StyleFormat *default_time_fmt;
+static StyleFormat *default_general_fmt;
+
+
 /* Points to the locale information for number display */
 static struct lconv *lc = NULL;
 static char *locale_currency = NULL; /* in UTF-8 */
@@ -240,12 +244,12 @@ append_year (GString *string, guchar const *format, struct tm const *time_split)
 	if ((format[2] != 'y' && format[2] != 'Y') ||
 	    (format[3] != 'y' && format[3] != 'Y')) {
 		sprintf (temp, "%02d", year % 100);
-		g_string_append (string, temp);
+		g_string_append_len (string, temp, 2);
 		return 2;
 	}
 
 	sprintf (temp, "%04d", year);
-	g_string_append (string, temp);
+	g_string_append_len (string, temp, 4);
 
 	return 4;
 }
@@ -267,7 +271,7 @@ append_month (GString *string, int n, struct tm const *time_split)
 
 	if (n == 2){
 		sprintf (temp, "%02d", time_split->tm_mon + 1);
-		g_string_append (string, temp);
+		g_string_append_len (string, temp, 2);
 		return 2;
 	}
 
@@ -297,7 +301,7 @@ append_day (GString *string, guchar const *format, struct tm const *time_split)
 
 	if (format[2] != 'd' && format[2] != 'D') {
 		sprintf (temp, "%02d", time_split->tm_mday);
-		g_string_append (string, temp);
+		g_string_append_len (string, temp, 2);
 		return 2;
 	}
 
@@ -311,9 +315,6 @@ append_day (GString *string, guchar const *format, struct tm const *time_split)
 	return 4;
 }
 
-/*
- * Renders the hour.
- */
 static void
 append_hour (GString *string, int n, struct tm const *time_split,
 	     gboolean want_am_pm)
@@ -327,21 +328,25 @@ append_hour (GString *string, int n, struct tm const *time_split,
 	g_string_append (string, temp);
 }
 
-/*
- * Renders the hour.
- */
 static void
-append_hour_elapsed (GString *string, struct tm const *time_split, int number)
+append_hour_elapsed (GString *string, struct tm *tm, gnm_float number)
 {
 	char buf[(DBL_MANT_DIG + DBL_MAX_EXP) * 2 + 1];
-	double hours = number * 24. + time_split->tm_hour;
-	snprintf (buf, sizeof (buf), "%.0f", hours);
+	double res, int_part;
+
+	/* ick.  round assuming no more than 100th of a second, we really need
+	 * to know the precision earlier */
+	res = gnumeric_fake_round (number * 24. * 60. * 60. * 100.);
+	res = modf (res / (60. * 60. * 100.), &int_part);
+	tm->tm_hour = int_part;
+	res *= ((res < 0.) ? -3600. : 3600.);
+	tm->tm_min = res / 60.;
+	tm->tm_sec = res - tm->tm_min * 60;
+
+	snprintf (buf, sizeof (buf), "%d", tm->tm_hour);
 	g_string_append (string, buf);
 }
 
-/*
- * Renders the number of minutes.
- */
 static void
 append_minute (GString *string, int n, struct tm const *time_split)
 {
@@ -350,15 +355,16 @@ append_minute (GString *string, int n, struct tm const *time_split)
 	g_string_append (string, temp);
 }
 
-/*
- * Renders the number of minutes, in elapsed format
- */
 static void
-append_minute_elapsed (GString *string, struct tm const *time_split, int number)
+append_minute_elapsed (GString *string, struct tm *tm, gnm_float number)
 {
 	char buf[(DBL_MANT_DIG + DBL_MAX_EXP) * 2 + 1];
-	double minutes = ((number * 24.) + time_split->tm_hour) * 60. + time_split->tm_min;
-	snprintf (buf, sizeof (buf), "%.0f", minutes);
+	double res, int_part;
+
+	res = modf (gnumeric_fake_round (number * 24. * 60.), &int_part);
+	tm->tm_min = int_part;
+	tm->tm_sec = res * ((res < 0.) ? -60. : 60.);
+	snprintf (buf, sizeof (buf), "%d", tm->tm_min);
 	g_string_append (string, buf);
 }
 
@@ -377,11 +383,11 @@ append_second (GString *string, int n, struct tm const *time_split)
  * Renders the second field in elapsed
  */
 static void
-append_second_elapsed (GString *string, int n, struct tm const *time_split, int number)
+append_second_elapsed (GString *string, gnm_float number)
 {
 	char buf[(DBL_MANT_DIG + DBL_MAX_EXP) * 2 + 1];
-	double seconds = (((number * 24. + time_split->tm_hour) * 60. + time_split->tm_min) * 60.) + time_split->tm_sec;
-	snprintf (buf, sizeof (buf), "%.0f", seconds);
+	snprintf (buf, sizeof (buf), "%d",
+		(int) gnumeric_fake_round (number * 24. * 3600.));
 	g_string_append (string, buf);
 }
 
@@ -582,7 +588,7 @@ format_compile (StyleFormat *format)
  *
  * resources allocated in format_compile should be disposed here
  */
-void
+static void
 format_destroy (StyleFormat *format)
 {
 	g_slist_foreach (format->entries, &format_entry_dtor, NULL);
@@ -621,8 +627,10 @@ format_color_shutdown (void)
 {
 	int i;
 
-	for (i = G_N_ELEMENTS (format_colors) ; i-- > 0 ; )
+	for (i = G_N_ELEMENTS (format_colors) ; i-- > 0 ; ) {
 		style_color_unref (format_colors[i].color);
+		format_colors[i].color = NULL;
+	}
 }
 
 static struct FormatColor const *
@@ -711,7 +719,7 @@ render_number (GString *result,
 			g_string_append_c (result, '0');
 	}
 
-	g_string_append (result, num);
+	g_string_append_len (result, num, num_buf + sizeof (num_buf) - 1 - num);
 
 	/* If the format contains only "#"s to the left of the decimal
 	 * point, number in the [0.0,1.0] range are prefixed with a
@@ -805,24 +813,24 @@ do_render_number (gnm_float number, format_info_t *info, GString *result)
  * > However, import or export date using serial date number will be a problem.
  * > If no one noticed anything wrong, it must be that no one did it that way.
  */
-static struct tm *
-split_time (gnm_float number)
+static gboolean
+split_time (struct tm *tm, gnm_float number, GnmDateConventions const *date_conv)
 {
-	static struct tm tm;
 	guint secs;
 	GDate date;
 
-	datetime_serial_to_g (&date, datetime_serial_raw_to_serial (number));
-	g_date_to_struct_tm (&date, &tm);
+	datetime_serial_to_g (&date,
+		datetime_serial_raw_to_serial (number), date_conv);
+	g_date_to_struct_tm (&date, tm);
 
 	secs = datetime_serial_raw_to_seconds (number);
-	tm.tm_hour = secs / 3600;
-	secs -= tm.tm_hour * 3600;
-	tm.tm_min  = secs / 60;
-	secs -= tm.tm_min * 60;
-	tm.tm_sec  = secs;
+	tm->tm_hour = secs / 3600;
+	secs -= tm->tm_hour * 3600;
+	tm->tm_min  = secs / 60;
+	secs -= tm->tm_min * 60;
+	tm->tm_sec  = secs;
 
-	return &tm;
+	return FALSE;
 }
 
 /*
@@ -894,7 +902,7 @@ reformat_decimals(FormatCharacteristics *fc,
 	fc->num_decimals += step;
 
 	/* Regenerate the format with the good function */
-	res = g_string_new ("");
+	res = g_string_new (NULL);
 	(*format_function) (res, fc);
 
 	return g_string_free (res, FALSE);
@@ -920,7 +928,7 @@ format_remove_decimal (StyleFormat const *fmt)
 	/* First try to classify the format so we can regenerate it */
 	ff = cell_format_classify (fmt, &fc);
 
-	switch (ff) {	
+	switch (ff) {
 	case FMT_NUMBER:
 	case FMT_CURRENCY:
 		return reformat_decimals (&fc, &style_format_number, -1);
@@ -996,7 +1004,7 @@ format_add_decimal (StyleFormat const *fmt)
 	/* First try to classify the format so we can regenerate it */
 	ff = cell_format_classify(fmt, &fc);
 
-	switch (ff) {	
+	switch (ff) {
 	case FMT_NUMBER:
 	case FMT_CURRENCY:
 		return reformat_decimals (&fc, &style_format_number, 1);
@@ -1019,7 +1027,7 @@ format_add_decimal (StyleFormat const *fmt)
 	}
 
 	/* Use the old code for more special formats to try to add a
-	   decimal */	
+	   decimal */
 
 	if (strcmp (format_string, "General") == 0) {
 		format_string = "0";
@@ -1066,21 +1074,23 @@ format_add_decimal (StyleFormat const *fmt)
 
 /*********************************************************************/
 
-static gchar *
-format_number (gnm_float number, int col_width, StyleFormatEntry const *entry)
+static void
+format_number (GString *result,
+	       gnm_float number, int col_width, StyleFormatEntry const *entry,
+	       GnmDateConventions const *date_conv)
 {
-	GString *result = g_string_new ("");
 	guchar const *format = (guchar *)(entry->format);
 	format_info_t info;
 	gboolean can_render_number = FALSE;
-	int hour_seen = 0;
+	gboolean hour_seen = FALSE;
 	gboolean time_display_elapsed = FALSE;
 	gboolean ignore_further_elapsed = FALSE;
 
 	gunichar fill_char = 0;
 	int fill_start = -1;
 
-	struct tm *time_split = 0;
+	gboolean need_time_split = TRUE;
+	struct tm tm;
 	gnm_float signed_number;
 
 	memset (&info, 0, sizeof (info));
@@ -1093,7 +1103,8 @@ format_number (gnm_float number, int col_width, StyleFormatEntry const *entry)
 	info.scale = 1;
 
 	while (*format) {
-		gunichar c = g_utf8_get_char (format);
+		/* This is just g_utf8_get_char, but we're in a hurry.  */
+		gunichar c = (*format & 0x80) ? g_utf8_get_char (format) : *format;
 
 		switch (c) {
 
@@ -1187,7 +1198,7 @@ format_number (gnm_float number, int col_width, StyleFormatEntry const *entry)
 				 prec, number);
 
 			g_string_append (result, buffer);
-			goto finish;
+			return;
 		}
 
 		case '\\':
@@ -1237,13 +1248,16 @@ format_number (gnm_float number, int col_width, StyleFormatEntry const *entry)
 						numerator = (int)((number - (int)number) * denominator + 0.5);
 					}
 				} else {
-					static int const powers[3] = { 10, 100, 1000 };
+					static int const powers[8] = {
+						10, 100, 1000, 10000, 100000,
+						1000000, 10000000, 100000000
+					};
 
 					format += size + 1;
-					if (size > 3)
-						size = 3;
-					stern_brocot (number - (int)number, powers[size - 1],
-						      &numerator, &denominator);
+					if (size > 8)
+						size = 8;
+					continued_fraction (number - (int)number, powers[size - 1],
+							    &numerator, &denominator);
 				}
 
 				if (denominator > 0) {
@@ -1322,9 +1336,6 @@ format_number (gnm_float number, int col_width, StyleFormatEntry const *entry)
 		case 'm': {
 			int n;
 
-			if (!time_split)
-				time_split = split_time (signed_number);
-
 			/* FIXME : Yuck
 			 * This is a problem waiting to happen.
 			 * rewrite.
@@ -1334,49 +1345,54 @@ format_number (gnm_float number, int col_width, StyleFormatEntry const *entry)
 			if (format[1] == ']')
 				format++;
 			if (time_display_elapsed) {
-				time_display_elapsed = FALSE;
+				need_time_split = time_display_elapsed = FALSE;
 				ignore_further_elapsed = TRUE;
-				append_minute_elapsed (result, time_split, number);
-			} else if (hour_seen ||
-				   (format[1] == ':' &&
-				    (format[2] == 's' || format[2] == 'S'))) {
-				append_minute (result, n, time_split);
+				append_minute_elapsed (result, &tm, signed_number);
+				break;
+			}
+
+			if (need_time_split)
+				need_time_split = split_time (&tm, signed_number, date_conv);
+			if (hour_seen ||
+			    (format[1] == ':' &&
+			     (format[2] == 's' || format[2] == 'S'))) {
+				append_minute (result, n, &tm);
 			} else
-				append_month (result, n, time_split);
+				append_month (result, n, &tm);
 			break;
 		}
 
 		case 'D':
 		case 'd':
-			if (!time_split)
-				time_split = split_time (signed_number);
-			format += append_day (result, format, time_split) - 1;
+			if (need_time_split)
+				need_time_split = split_time (&tm, signed_number, date_conv);
+			format += append_day (result, format, &tm) - 1;
 			break;
 
 		case 'Y':
 		case 'y':
-			if (!time_split)
-				time_split = split_time (signed_number);
-			format += append_year (result, format, time_split) - 1;
+			if (need_time_split)
+				need_time_split = split_time (&tm, signed_number, date_conv);
+			format += append_year (result, format, &tm) - 1;
 			break;
 
 		case 'S':
 		case 's': {
 			int n;
 
-			if (!time_split)
-				time_split = split_time (signed_number);
-
 			for (n = 1; format[1] == 's' || format[1] == 'S'; format++)
 				n++;
 			if (format[1] == ']')
 				format++;
-			if (time_display_elapsed){
-				time_display_elapsed = FALSE;
+			if (time_display_elapsed) {
+				need_time_split = time_display_elapsed = FALSE;
 				ignore_further_elapsed = TRUE;
-				append_second_elapsed (result, n, time_split, number);
-			} else
-				append_second (result, n, time_split);
+				append_second_elapsed (result, signed_number);
+			} else {
+				if (need_time_split)
+					need_time_split = split_time (&tm, signed_number, date_conv);
+				append_second (result, n, &tm);
+			}
 			break;
 		}
 
@@ -1384,35 +1400,36 @@ format_number (gnm_float number, int col_width, StyleFormatEntry const *entry)
 		case 'h': {
 			int n;
 
-			if (!time_split)
-				time_split = split_time (signed_number);
-
 			for (n = 1; format[1] == 'h' || format[1] == 'H'; format++)
 				n++;
 			if (format[1] == ']')
 				format++;
-			if (time_display_elapsed){
-				time_display_elapsed = FALSE;
+			if (time_display_elapsed) {
+				need_time_split = time_display_elapsed = FALSE;
 				ignore_further_elapsed = TRUE;
-				append_hour_elapsed (result, time_split, number);
-			} else
+				append_hour_elapsed (result, &tm, signed_number);
+			} else {
 				/* h == hour optionally in 24 hour mode
-				 * h followed by am/pm puts it in 12 hout mode
+				 * h followed by am/pm puts it in 12 hour mode
 				 *
 				 * multiple h eg 'hh' force 12 hour mode.
 				 * NOTE : This is a non-XL extension
 				 */
-				append_hour (result, n, time_split,
+				if (need_time_split)
+					need_time_split = split_time (&tm, signed_number, date_conv);
+
+				append_hour (result, n, &tm,
 					     entry->want_am_pm || (n > 1));
+			}
 			hour_seen = TRUE;
 			break;
 		}
 
 		case 'A':
 		case 'a':
-			if (!time_split)
-				time_split = split_time (signed_number);
-			if (time_split->tm_hour < 12){
+			if (need_time_split)
+				need_time_split = split_time (&tm, signed_number, date_conv);
+			if (tm.tm_hour < 12){
 				g_string_append_c (result, *format);
 				format++;
 				if (*format == 'm' || *format == 'M'){
@@ -1429,9 +1446,9 @@ format_number (gnm_float number, int col_width, StyleFormatEntry const *entry)
 			break;
 
 		case 'P': case 'p':
-			if (!time_split)
-				time_split = split_time (signed_number);
-			if (time_split->tm_hour >= 12){
+			if (need_time_split)
+				need_time_split = split_time (&tm, signed_number, date_conv);
+			if (tm.tm_hour >= 12){
 				g_string_append_c (result, *format);
 				if (*(format + 1) == 'm' || *(format + 1) == 'M'){
 					format++;
@@ -1461,9 +1478,6 @@ format_number (gnm_float number, int col_width, StyleFormatEntry const *entry)
 		while (count-- > 0)
 			g_string_insert_unichar (result, fill_start, fill_char);
 	}
-
- finish:
-	return g_string_free (result, FALSE);
 }
 
 static gboolean
@@ -1512,14 +1526,16 @@ style_format_condition (StyleFormatEntry const *entry, Value const *value)
  * @val : the integer value being formated.
  * @col_width : the approximate width in characters.
  */
-static char *
-fmt_general_float (gnm_float val, double col_width)
+static void
+fmt_general_float (GString *result, gnm_float val, double col_width)
 {
 	gnm_float tmp;
 	int log_val, prec;
 
-	if (col_width < 0.)
-		return g_strdup_printf ("%.*" GNUM_FORMAT_g, GNUM_DIG, val);
+	if (col_width < 0.) {
+		g_string_append_printf (result, "%.*" GNUM_FORMAT_g, GNUM_DIG, val);
+		return;
+	}
 
 	if (val < 0.) {
 		/* leave space for minus sign */
@@ -1547,8 +1563,10 @@ fmt_general_float (gnm_float val, double col_width)
 
 		/* Display 0 for cols that are too narrow for scientific
 		 * notation with abs (value) < 1 */
-		if (col_width < 5. && -log_val >= prec)
-			return g_strdup ("0");
+		if (col_width < 5. && -log_val >= prec) {
+			g_string_append_c (result, '0');
+			return;
+		}
 
 		/* Include leading zeros eg 0.0x has 2 leading zero */
 		if (log_val >= -4)
@@ -1565,7 +1583,7 @@ fmt_general_float (gnm_float val, double col_width)
 		prec = GNUM_DIG;
 
 	/* FIXME : glib bug.  it does not handle G, use g (fixed in 1.2.9) */
-	return g_strdup_printf ("%.*" GNUM_FORMAT_g, prec, val);
+	g_string_append_printf (result, "%.*" GNUM_FORMAT_g, prec, val);
 }
 
 /**
@@ -1574,8 +1592,8 @@ fmt_general_float (gnm_float val, double col_width)
  * @val : the integer value being formated.
  * @col_width : the approximate width in characters.
  */
-static char *
-fmt_general_int (int val, int col_width)
+static void
+fmt_general_int (GString *result, int val, int col_width)
 {
 	if (col_width > 0) {
 		int log_val;
@@ -1588,22 +1606,26 @@ fmt_general_int (int val, int col_width)
 			log_val = (val > 0) ? ceil (log10 (val)) : 0;
 
 		/* Switch to scientific notation if things are too wide */
-		if (log_val > col_width)
+		if (log_val > col_width) {
 			/* FIXME : glib bug.  it does not handle G, use g */
 			/* Decrease available width by 5 to account for .+E00 */
-			return g_strdup_printf ("%.*g", col_width - 5, (double)val);
+			g_string_append_printf (result, "%.*g", col_width - 5, (double)val);
+			return;
+		}
 	}
-	return g_strdup_printf ("%d", val);
+
+	/* FIXME: we can do better than this.  */
+	g_string_append_printf (result, "%d", val);
 }
 
 /*
  * Returns NULL when the value should be formated as text
  */
-gchar *
-format_value (StyleFormat const *format, Value const *value, StyleColor **color,
-	      double col_width)
+void
+format_value_gstring (GString *result, StyleFormat const *format,
+		      Value const *value, StyleColor **color,
+		      double col_width, GnmDateConventions const *date_conv)
 {
-	char *v = NULL;
 	StyleFormatEntry const *entry = NULL; /* default to General */
 	GSList *list;
 	gboolean need_abs = FALSE;
@@ -1611,7 +1633,7 @@ format_value (StyleFormat const *format, Value const *value, StyleColor **color,
 	if (color)
 		*color = NULL;
 
-	g_return_val_if_fail (value != NULL, g_strdup ("<ERROR>"));
+	g_return_if_fail (value != NULL);
 
 	if (format == NULL)
 		format = VALUE_FMT (value);
@@ -1634,7 +1656,7 @@ format_value (StyleFormat const *format, Value const *value, StyleColor **color,
 
 			/* Empty formats should be ignored */
 			if (entry->format[0] == '\0')
-				return g_strdup ("");
+				return;
 
 			if (color && entry->color != NULL)
 				*color = style_color_ref (entry->color);
@@ -1654,56 +1676,70 @@ format_value (StyleFormat const *format, Value const *value, StyleColor **color,
 
 	switch (value->type) {
 	case VALUE_EMPTY:
-		return g_strdup ("");
+		return;
 	case VALUE_BOOLEAN:
-		return g_strdup (value->v_bool.val ? _("TRUE"):_("FALSE"));
+		g_string_append (result, value->v_bool.val ? _("TRUE"):_("FALSE"));
+		return;
 	case VALUE_INTEGER: {
 		int val = value->v_int.val;
 		if (need_abs)
 			val = ABS (val);
 
 		if (entry == NULL)
-			return fmt_general_int (val, col_width);
-		v = format_number (val, (int)col_width, entry);
-		break;
+			fmt_general_int (result, val, col_width);
+		else
+			format_number (result, val, (int)col_width, entry, date_conv);
+		return;
 	}
 	case VALUE_FLOAT: {
 		gnm_float val = value->v_float.val;
 
-		if (!finitegnum (val))
-			return g_strdup (value_error_name (GNM_ERROR_VALUE, TRUE));
+		if (!finitegnum (val)) {
+			g_string_append (result, value_error_name (GNM_ERROR_VALUE, TRUE));
+			return;
+		}
 
 		if (need_abs)
 			val = gnumabs (val);
 
 		if (entry == NULL) {
 			if (INT_MAX >= val && val >= INT_MIN && val == floorgnum (val))
-				return fmt_general_int ((int)val, col_width);
-			return fmt_general_float (val, col_width);
-		}
-		v = format_number (val, (int)col_width, entry);
-		break;
+				fmt_general_int (result, (int)val, col_width);
+			else
+				fmt_general_float (result, val, col_width);
+		} else
+			format_number (result, val, (int)col_width, entry, date_conv);
+		return;
 	}
 	case VALUE_ERROR:
-		return g_strdup (value->v_err.mesg->str);
+		g_string_append (result, value->v_err.mesg->str);
+		return;
 	case VALUE_STRING:
-		return g_strdup (value->v_str.val->str);
+		g_string_append (result, value->v_str.val->str);
+		return;
 	case VALUE_CELLRANGE:
-		return g_strdup (value_error_name (GNM_ERROR_VALUE, TRUE));
+		g_string_append (result, value_error_name (GNM_ERROR_VALUE, TRUE));
+		return;
 	case VALUE_ARRAY: /* Array of arrays ?? */
-		return g_strdup (_("ARRAY"));
+		g_string_append (result, _("ARRAY"));
+		return;
 
 	default:
-		return g_strdup ("Internal error");
+		g_assert_not_reached ();
+		return;
 	}
-
-	/* Format error, return a default value */
-	if (v == NULL)
-		return value_get_as_string (value);
-
-	return v;
 }
 
+gchar *
+format_value (StyleFormat const *format, Value const *value, StyleColor **color,
+	      double col_width, GnmDateConventions const *date_conv)
+{
+	GString *result = g_string_sized_new (20);
+	format_value_gstring (result, format, value, color, col_width, date_conv);
+	return g_string_free (result, FALSE);
+}
+
+
 void
 number_format_init (void)
 {
@@ -1712,9 +1748,44 @@ number_format_init (void)
 	beyond_precision = gpow10 (GNUM_DIG + 1);
 }
 
+static void
+cb_format_leak (gpointer key, gpointer value, gpointer user_data)
+{
+	StyleFormat *format = value;
+
+	fprintf (stderr, "Leaking style-format at %p [%s].\n",
+		 format, format->format);
+}
+
 void
 number_format_shutdown (void)
 {
+	if (default_percentage_fmt) {
+		style_format_unref (default_percentage_fmt);
+		default_percentage_fmt = NULL;
+	}
+
+	if (default_money_fmt) {
+		style_format_unref (default_money_fmt);
+		default_money_fmt = NULL;
+	}
+
+	if (default_date_fmt) {
+		style_format_unref (default_date_fmt);
+		default_date_fmt = NULL;
+	}
+
+	if (default_time_fmt) {
+		style_format_unref (default_time_fmt);
+		default_time_fmt = NULL;
+	}
+
+	if (default_general_fmt) {
+		style_format_unref (default_general_fmt);
+		default_general_fmt = NULL;
+	}
+
+	g_hash_table_foreach (style_format_hash, cb_format_leak, NULL);
 	g_hash_table_destroy (style_format_hash);
 	style_format_hash = NULL;
 }
@@ -1797,6 +1868,8 @@ style_format_delocalize (char const *descriptor_string)
  *
  * Looks up and potentially creates a StyleFormat from the supplied string in
  * XL format.
+ *
+ * @descriptor_string: XL descriptor in UTF-8 encoding.
  */
 StyleFormat *
 style_format_new_XL (char const *descriptor_string, gboolean delocalize)
@@ -1988,43 +2061,49 @@ style_format_is_text (StyleFormat const *sf)
 StyleFormat *
 style_format_general (void)
 {
-	static StyleFormat *fmt = NULL;
-	if (fmt == NULL)
-		fmt = style_format_new_XL (cell_formats[FMT_GENERAL][0], FALSE);
-	return fmt;
+	if (!default_general_fmt)
+		default_general_fmt =
+			style_format_new_XL (cell_formats[FMT_GENERAL][0], FALSE);
+
+	return default_general_fmt;
 }
+
 StyleFormat *
 style_format_default_date (void)
 {
-	static StyleFormat *fmt = NULL;
-	if (fmt == NULL)
-		fmt = style_format_new_XL (cell_formats[FMT_DATE][0], FALSE);
-	return fmt;
+	if (!default_date_fmt)
+		default_date_fmt =
+			style_format_new_XL (cell_formats[FMT_DATE][0], FALSE);
+
+	return default_date_fmt;
 }
 
 StyleFormat *
 style_format_default_time (void)
 {
-	static StyleFormat *fmt = NULL;
-	if (fmt == NULL)
-		fmt = style_format_new_XL (cell_formats[FMT_TIME][0], FALSE);
-	return fmt;
+	if (!default_time_fmt)
+		default_time_fmt =
+			style_format_new_XL (cell_formats[FMT_TIME][0], FALSE);
+
+	return default_time_fmt;
 }
 
 StyleFormat *
 style_format_default_percentage	(void)
 {
-	static StyleFormat *fmt = NULL;
-	if (fmt == NULL)
-		fmt = style_format_new_XL (cell_formats[FMT_PERCENT][1], FALSE);
-	return fmt;
+	if (!default_percentage_fmt)
+		default_percentage_fmt =
+			style_format_new_XL (cell_formats[FMT_PERCENT][1], FALSE);
+
+	return default_percentage_fmt;
 }
 
 StyleFormat *
 style_format_default_money (void)
 {
-	static StyleFormat *fmt = NULL;
-	if (fmt == NULL)
-		fmt = style_format_new_XL (cell_formats[FMT_CURRENCY][3], FALSE);
-	return fmt;
+	if (!default_money_fmt)
+		default_money_fmt =
+			style_format_new_XL (cell_formats[FMT_CURRENCY][2], FALSE);
+
+	return default_money_fmt;
 }
