@@ -14,7 +14,6 @@
 #include "sheet-control-gui-priv.h"
 #include "gui-util.h"
 #include "mstyle.h"
-#include "style-color.h"
 #include "selection.h"
 #include "parse-util.h"
 #include "ranges.h"
@@ -32,6 +31,7 @@
 #include "clipboard.h"
 #include <gsf/gsf-impl-utils.h>
 #include <gdk/gdkkeysyms.h>
+#include <string.h>
 
 static FooCanvasClass *parent_klass;
 
@@ -287,7 +287,7 @@ gnm_canvas_key_mode_sheet (GnmCanvas *gcanvas, GdkEventKey *event)
 		if (wbcg_is_editing (wbcg))
 			sheet = wbcg->wb_control.editing_sheet;
 
-		if (wbcg_edit_finish (wbcg, TRUE)) {
+		if (wbcg_edit_finish (wbcg, TRUE, NULL)) {
 			/* Figure out the direction */
 			gboolean const direction = (event->state & GDK_SHIFT_MASK) ? FALSE : TRUE;
 			gboolean const horizontal = (event->keyval == GDK_KP_Enter ||
@@ -298,7 +298,7 @@ gnm_canvas_key_mode_sheet (GnmCanvas *gcanvas, GdkEventKey *event)
 		break;
 
 	case GDK_Escape:
-		wbcg_edit_finish (wbcg, FALSE);
+		wbcg_edit_finish (wbcg, FALSE, NULL);
 		application_clipboard_unant ();
 		break;
 
@@ -487,9 +487,9 @@ gnm_canvas_realize (GtkWidget *w)
 	/* Set the default background color of the canvas itself to white.
 	 * This makes the redraws when the canvas scrolls flicker less. */
 	style = gtk_style_copy (w->style);
-	style->bg [GTK_STATE_NORMAL] = gs_white;
+	style->bg[GTK_STATE_NORMAL] = style->white;
 	gtk_widget_set_style (w, style);
-	gtk_style_unref (style);
+	g_object_unref (style);
 
 	gtk_im_context_set_client_window (GNM_CANVAS (w)->im_context,
 		gtk_widget_get_toplevel (w)->window);
@@ -974,7 +974,7 @@ gnm_canvas_slide_stop (GnmCanvas *gcanvas)
 	if (gcanvas->sliding == -1)
 		return;
 
-	gtk_timeout_remove (gcanvas->sliding);
+	g_source_remove (gcanvas->sliding);
 	gcanvas->slide_handler = NULL;
 	gcanvas->slide_data = NULL;
 	gcanvas->sliding = -1;
@@ -1149,7 +1149,7 @@ gcanvas_sliding_callback (gpointer data)
 
 	if (slide_x || slide_y) {
 		if (gcanvas->sliding == -1)
-			gcanvas->sliding = gtk_timeout_add (
+			gcanvas->sliding = g_timeout_add (
 				300, gcanvas_sliding_callback, gcanvas);
 	} else
 		gnm_canvas_slide_stop (gcanvas);
