@@ -5,6 +5,9 @@
  * Author:
  *  Miguel de Icaza (miguel@gnu.org)
  */
+
+#undef GTK_DISABLE_DEPRECATED
+#warning "This file uses GTK_DISABLE_DEPRECATED"
 #include <gnumeric-config.h>
 #include <gnumeric-i18n.h>
 #include "gnumeric.h"
@@ -21,6 +24,7 @@
 #include "format.h"
 #include "application.h"
 #include "workbook.h"
+#include "gnumeric-paths.h"
 
 #include <string.h>
 #include <gal/widgets/widget-color-combo.h>
@@ -296,6 +300,7 @@ gnumeric_error_info_dialog_new (ErrorInfo *error)
 	gchar *message;
 	gint bf_lim = 1;
 	gint i;
+	GdkScreen *screen;
 
 	g_return_val_if_fail (error != NULL, NULL);
 
@@ -308,7 +313,10 @@ gnumeric_error_info_dialog_new (ErrorInfo *error)
 		mtype = GTK_MESSAGE_WARNING;
 	dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
 					 mtype, GTK_BUTTONS_CLOSE, " ");
-	gtk_widget_set_usize (dialog, 450, 250);
+	screen = gtk_widget_get_screen (dialog);
+	gtk_widget_set_size_request (dialog,
+				     gdk_screen_get_width (screen) / 3,
+				     gdk_screen_get_width (screen) / 4);
 	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
 					GTK_POLICY_AUTOMATIC,
@@ -341,6 +349,7 @@ gnumeric_error_info_dialog_new (ErrorInfo *error)
 	gtk_widget_show_all (GTK_WIDGET (scrolled_window));
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), scrolled_window, TRUE, TRUE, 0);
 
+	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CLOSE);
 	return dialog;
 }
 
@@ -414,8 +423,12 @@ cb_free_keyed_dialog_context (KeyedDialogContext *ctxt)
 	ctxt->freed = TRUE;
 
 	g_return_if_fail (g_object_get_data (G_OBJECT (ctxt->wbcg), ctxt->key));
+	/*
+	 * One of these causes a recursive call which will do nothing due to
+	 * ->freed.
+	 */
 	g_object_set_data (G_OBJECT (ctxt->wbcg), ctxt->key, NULL);
-	g_object_set_data (G_OBJECT (ctxt->wbcg), "KeyedDialog", NULL);
+	g_object_set_data (G_OBJECT (ctxt->dialog), "KeyedDialog", NULL);
 	g_free (ctxt);
 }
 
@@ -676,7 +689,7 @@ gnumeric_popup_menu (GtkMenu *menu, GdkEventButton *event)
 	g_return_if_fail (menu != NULL);
 	g_return_if_fail (GTK_IS_MENU (menu));
 
-	gtk_object_ref (GTK_OBJECT (menu));
+	g_object_ref (menu);
 	gtk_object_sink (GTK_OBJECT (menu));
 
 	g_signal_connect (G_OBJECT (menu),
@@ -724,7 +737,7 @@ cb_clist_moveto (GtkWidget *clist, gpointer row)
  * mapped.
  */
 void
-gnumeric_clist_moveto (GtkCList *clist, gint row)
+gnumeric_clist_moveto (/*GtkCList*/ void *clist, gint row)
 {
 	if (GTK_WIDGET_DRAWABLE (clist))
 		clist_moveto (clist, row);
@@ -772,19 +785,21 @@ gnumeric_position_tooltip (GtkWidget *tip, int horizontal)
 
 	gtk_widget_size_request (tip, &req);
 	gdk_window_get_pointer (NULL, &x, &y, NULL);
+
 	if (horizontal){
-		x = x - req.width/2;
-		y = y - req.height - 20;
+		x -= req.width / 2;
+		y -= req.height + 20;
 	} else {
-		x = x - req.width - 20;
-		y = y - req.height/2;
+		x -= req.width + 20;
+		y -= req.height / 2;
 	}
 
 	if (x < 0)
 		x = 0;
 	if (y < 0)
 		y = 0;
-	gtk_widget_set_uposition (gtk_widget_get_toplevel (tip), x, y);
+
+	gtk_window_move (GTK_WINDOW (gtk_widget_get_toplevel (tip)), x, y);
 }
 
 /**
@@ -873,9 +888,9 @@ static void
 popup_item_activate (GtkWidget *item, gpointer *user_data)
 {
 	GnumericPopupMenuElement const *elem =
-		gtk_object_get_data (GTK_OBJECT (item), "descriptor");
+		g_object_get_data (G_OBJECT (item), "descriptor");
 	GnumericPopupMenuHandler handler =
-		gtk_object_get_data (GTK_OBJECT (item), "handler");
+		g_object_get_data (G_OBJECT (item), "handler");
 
 	g_return_if_fail (elem != NULL);
 	g_return_if_fail (handler != NULL);
@@ -932,14 +947,14 @@ gnumeric_create_popup_menu_list (GSList *elements,
 			g_signal_connect (G_OBJECT (item),
 				"activate",
 				G_CALLBACK (&popup_item_activate), user_data);
-			gtk_object_set_data (
-				GTK_OBJECT (item), "descriptor", (gpointer)(element));
-			gtk_object_set_data (
-				GTK_OBJECT (item), "handler", (gpointer)handler);
+			g_object_set_data (
+				G_OBJECT (item), "descriptor", (gpointer)(element));
+			g_object_set_data (
+				G_OBJECT (item), "handler", (gpointer)handler);
 		}
 
 		gtk_widget_show (item);
-		gtk_menu_append (GTK_MENU (menu), item);
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 	}
 
 	gnumeric_popup_menu (GTK_MENU (menu), event);
@@ -1112,7 +1127,7 @@ focus_on_entry (GtkEntry *entry)
 		return;
 	gtk_widget_grab_focus (GTK_WIDGET(entry));
 	gtk_editable_set_position (GTK_EDITABLE (entry), 0);
-	gtk_entry_select_region (entry, 0, entry->text_length);
+	gtk_editable_select_region (GTK_EDITABLE (entry), 0, entry->text_length);
 }
 
 gboolean
@@ -1279,3 +1294,571 @@ gnm_setup_label_atk (GtkLabel *label, GtkWidget *target)
 	 add_atk_relation (GTK_WIDGET (label), target, ATK_RELATION_LABEL_FOR);
 	 add_atk_relation (target, GTK_WIDGET (label), ATK_RELATION_LABELLED_BY);
 }
+
+
+int
+gnm_measure_string (PangoContext *context, const PangoFontDescription *font_desc, const char *str)
+{
+	PangoLayout *layout = pango_layout_new (context);
+	int width;
+
+	pango_layout_set_text (layout, str, -1);
+	pango_layout_set_font_description (layout, font_desc);
+	pango_layout_get_pixel_size (layout, &width, NULL);
+
+	g_object_unref (layout);
+
+	return width;
+}
+
+static void
+cb_focus_to_entry (GtkWidget *button, GtkWidget *entry)
+{
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)))
+		gtk_widget_grab_focus (entry);
+}
+
+static gboolean
+cb_activate_button (G_GNUC_UNUSED GtkWidget *widget,
+		    G_GNUC_UNUSED GdkEventFocus *event,
+		    GtkWidget *button)
+{
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
+	return FALSE;
+}
+
+void
+gnm_link_button_and_entry (GtkWidget *button, GtkWidget *entry)
+{
+	g_signal_connect (G_OBJECT (button),
+			  "clicked", G_CALLBACK (cb_focus_to_entry),
+			  entry);
+	g_signal_connect (G_OBJECT (entry),
+			  "focus_in_event",
+			  G_CALLBACK (cb_activate_button),
+			  button);
+}
+
+/* ------------------------------------------------------------------------- */
+/* Refugees from GAL.  */
+
+#define GDK_INTERNAL_CURSOR -1
+
+typedef struct {
+	GdkCursor *cursor;
+	int       hot_x, hot_y;
+	const char **xpm;
+	GdkBitmap *bitmap, *mask;
+	GdkDrawable *last_drawable;
+} CursorDef;
+
+/* Hack so we don't have to change the xpms below.  */
+#define char const char
+
+/* XPM */
+static char * cursor_cross_xpm[] = {
+"32 32 3 1",
+" 	c None",
+".	c #000000",
+"+	c #FFFFFF",
+"                                ",
+"                                ",
+"                                ",
+"                                ",
+"                                ",
+"                                ",
+"                                ",
+"             .......            ",
+"             .+++++..           ",
+"             .+++++..           ",
+"             .+++++..           ",
+"             .+++++..           ",
+"        ......+++++......       ",
+"        .+++++++++++++++..      ",
+"        .+++++++++++++++..      ",
+"        .+++++++++++++++..      ",
+"        .+++++++++++++++..      ",
+"        .+++++++++++++++..      ",
+"        ......+++++.......      ",
+"         .....+++++.......      ",
+"             .+++++..           ",
+"             .+++++..           ",
+"             .+++++..           ",
+"             ........           ",
+"              .......           ",
+"                                ",
+"                                ",
+"                                ",
+"                                ",
+"                                ",
+"                                ",
+"                                "};
+
+
+/* XPM */
+static char * cursor_zoom_in_xpm[] = {
+"32 32 2 1",
+" 	c None",
+".	c #000000",
+"                                ",
+"                                ",
+"                                ",
+"                                ",
+"                                ",
+"                                ",
+"        ......                  ",
+"       ..     ..                ",
+"      ..       ..               ",
+"      .         .               ",
+"     .    ...    .              ",
+"     .    ...    .              ",
+"     .  .......  .              ",
+"     .  .......  .              ",
+"     .    ...    .              ",
+"      .   ...   ..              ",
+"      ..       . .              ",
+"       ..     . . .             ",
+"        ........ . .            ",
+"         .....  . . .           ",
+"                 . . .          ",
+"                  . . .         ",
+"                   . . .        ",
+"                    . . .       ",
+"                     .   .      ",
+"                      .  .      ",
+"                       ..       ",
+"                                ",
+"                                ",
+"                                ",
+"                                ",
+"                                "};
+
+/* XPM */
+static char * cursor_zoom_out_xpm[] = {
+"32 32 2 1",
+" 	c None",
+".	c #000000",
+"                                ",
+"                                ",
+"                                ",
+"                                ",
+"                                ",
+"                                ",
+"        ......                  ",
+"       ..     ..                ",
+"      ..       ..               ",
+"      .         .               ",
+"     .           .              ",
+"     .           .              ",
+"     .  .......  .              ",
+"     .  .......  .              ",
+"     .           .              ",
+"      .         ..              ",
+"      ..       . .              ",
+"       ..     . . .             ",
+"        ........ . .            ",
+"         .....  . . .           ",
+"                 . . .          ",
+"                  . . .         ",
+"                   . . .        ",
+"                    . . .       ",
+"                     .   .      ",
+"                      .  .      ",
+"                       ..       ",
+"                                ",
+"                                ",
+"                                ",
+"                                ",
+"                                "};
+
+/* XPM */
+static char * cursor_hand_open_xpm[] = {
+"32 32 3 1",
+" 	c None",
+".	c #000000",
+"+	c #FFFFFF",
+"                                ",
+"                                ",
+"         ..                     ",
+"     .. .++...                  ",
+"    .++..++.++.                 ",
+"    .++..++.++. .               ",
+"     .++.++.++..+.              ",
+"     .++.++.++.++.              ",
+"   .. .+++++++.++.              ",
+"  .++..++++++++++.              ",
+"  .+++.+++++++++.               ",
+"   .++++++++++++.               ",
+"    .+++++++++++.               ",
+"    .++++++++++.                ",
+"     .+++++++++.                ",
+"      .+++++++.                 ",
+"       .++++++.                 ",
+"       .++++++.                 ",
+"                                ",
+"                                ",
+"                                ",
+"                                ",
+"                                ",
+"                                ",
+"                                ",
+"                                ",
+"                                ",
+"                                ",
+"                                ",
+"                                ",
+"                                ",
+"                                "};
+
+
+/* XPM */
+static char * cursor_hand_closed_xpm[] = {
+"32 32 3 1",
+" 	c None",
+".	c #000000",
+"+	c #FFFFFF",
+"                                ",
+"                                ",
+"                                ",
+"                                ",
+"                                ",
+"         ..                     ",
+"       ..++...                  ",
+"      .+++++++..                ",
+"      .+++++++++.               ",
+"    ...+++++++++.               ",
+"   .++.+++++++++.               ",
+"   .++++++++++++.               ",
+"    .+++++++++++.               ",
+"    .++++++++++.                ",
+"     .+++++++++.                ",
+"      .+++++++.                 ",
+"       .++++++.                 ",
+"       .++++++.                 ",
+"                                ",
+"                                ",
+"                                ",
+"                                ",
+"                                ",
+"                                ",
+"                                ",
+"                                ",
+"                                ",
+"                                ",
+"                                ",
+"                                ",
+"                                ",
+"                                "};
+
+#undef char
+
+static CursorDef
+cursors [] = {
+	{ NULL, 17, 17, cursor_cross_xpm },
+	{ NULL, GDK_INTERNAL_CURSOR,   GDK_CROSSHAIR,         NULL },
+	{ NULL, GDK_INTERNAL_CURSOR,   GDK_ARROW,             NULL },
+	{ NULL, GDK_INTERNAL_CURSOR,   GDK_FLEUR,             NULL },
+	{ NULL, 24, 24, cursor_zoom_in_xpm },
+	{ NULL, 24, 24, cursor_zoom_out_xpm },
+	{ NULL, GDK_INTERNAL_CURSOR,   GDK_SB_H_DOUBLE_ARROW, NULL },
+	{ NULL, GDK_INTERNAL_CURSOR,   GDK_SB_V_DOUBLE_ARROW, NULL },
+	{ NULL, GDK_INTERNAL_CURSOR,   GDK_SIZING,            NULL },
+	{ NULL, GDK_INTERNAL_CURSOR,   GDK_SIZING,            NULL },
+	{ NULL, GDK_INTERNAL_CURSOR,   GDK_HAND2,             NULL },
+	{ NULL, 10, 10, cursor_hand_open_xpm },
+	{ NULL, 10, 10, cursor_hand_closed_xpm },
+	{ NULL, GDK_INTERNAL_CURSOR,   GDK_XTERM,             NULL },
+	{ NULL, 0,    0,  NULL }
+};
+
+static void
+gnm_cursor_init1 (GdkDrawable *drawable, int i)
+{
+	int height, width, colors;
+	char pixmap_buffer [(32 * 32)/8];
+	char mask_buffer [(32 * 32)/8];
+	int x, y, pix, yofs;
+	int transparent_color, black_color;
+	const char **xpm = cursors[i].xpm;
+
+	sscanf (xpm [0], "%d %d %d %d", &height, &width, &colors, &pix);
+
+	g_assert (height == 32);
+	g_assert (width  == 32);
+	g_assert (colors <= 3);
+
+	transparent_color = ' ';
+	black_color = '.';
+	
+	yofs = colors + 1;
+	for (y = 0; y < 32; y++){
+		for (x = 0; x < 32;){
+			char value = 0, maskv = 0;
+			
+			for (pix = 0; pix < 8; pix++, x++){
+				if (xpm [y + yofs][x] != transparent_color){
+					maskv |= 1 << pix;
+
+					/*
+					 * Invert the colours here because it seems
+					 * to workaround a bug the Matrox G100 Xserver?
+					 * We reverse the foreground & background in the next
+					 * routine to compensate.
+					 */
+					if (xpm [y + yofs][x] == black_color){
+						value |= 1 << pix;
+					}
+				}
+			}
+			pixmap_buffer [(y * 4 + x/8)-1] = value;
+			mask_buffer [(y * 4 + x/8)-1] = maskv;
+		}
+	}
+
+	cursors[i].bitmap = gdk_bitmap_create_from_data (drawable, pixmap_buffer, 32, 32);
+	cursors[i].mask   = gdk_bitmap_create_from_data (drawable, mask_buffer, 32, 32);
+}
+
+static GdkCursor *
+gnm_cursor_create (GdkDrawable *drawable, GnmCursorType c)
+{
+	g_return_val_if_fail (c >= 0 && c < GNM_CURSOR_NUM_CURSORS, NULL);
+
+	if (cursors[c].hot_x == GDK_INTERNAL_CURSOR) {
+		GdkDisplay *display = gdk_drawable_get_display (drawable);
+		GdkCursorType typ = cursors[c].hot_y;
+		return gdk_cursor_new_for_display (display, typ);
+	} else {
+		if (drawable != cursors[c].last_drawable) {
+			if (cursors[c].bitmap)
+				g_object_unref (cursors[c].bitmap);
+			if (cursors[c].mask)
+				g_object_unref (cursors[c].mask);
+			gnm_cursor_init1 (drawable, c);
+			cursors[c].last_drawable = drawable;
+		}
+
+		return gdk_cursor_new_from_pixmap (
+			cursors[c].bitmap,
+			cursors[c].mask,
+			&gs_black, &gs_white,
+			cursors[c].hot_x,
+			cursors[c].hot_y);
+	}
+}
+
+void
+gnm_cursor_set_widget (GtkWidget *w, GnmCursorType c)
+{
+	if (w->window) {
+		GdkCursor *cursor = gnm_cursor_create (w->window, c);
+		gdk_window_set_cursor (w->window, cursor);
+		gdk_cursor_unref (cursor);
+	}
+}
+
+/* ------------------------------------------------------------------------- */
+
+/**
+ * gnumeric_button_new_with_stock_image
+ *
+ * Code from gedit
+ *
+ * Creates a new GtkButton with custom label and stock image.
+ * 
+ * text : button label
+ * sotck_id : id for stock icon
+ *
+ * return : newly created button
+ *
+ **/
+
+GtkWidget* 
+gnumeric_button_new_with_stock_image (const gchar* text, const gchar* stock_id)
+{
+	GtkWidget *button;
+	GtkStockItem item;
+	GtkWidget *label;
+	GtkWidget *image;
+	GtkWidget *hbox;
+	GtkWidget *align;
+
+	button = gtk_button_new ();
+
+	if (GTK_BIN (button)->child)
+		gtk_container_remove (GTK_CONTAINER (button),
+				      GTK_BIN (button)->child);
+
+	if (gtk_stock_lookup (stock_id, &item)) {
+		label = gtk_label_new_with_mnemonic (text);
+
+		gtk_label_set_mnemonic_widget (GTK_LABEL (label), GTK_WIDGET (button));
+
+		image = gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_BUTTON);
+		hbox = gtk_hbox_new (FALSE, 2);
+
+		align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
+
+		gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
+		gtk_box_pack_end (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+
+		gtk_container_add (GTK_CONTAINER (button), align);
+		gtk_container_add (GTK_CONTAINER (align), hbox);
+		gtk_widget_show_all (align);
+
+		return button;
+	}
+
+	label = gtk_label_new_with_mnemonic (text);
+	gtk_label_set_mnemonic_widget (GTK_LABEL (label), GTK_WIDGET (button));
+
+	gtk_misc_set_alignment (GTK_MISC (label), 0.5, 0.5);
+
+	gtk_widget_show (label);
+	gtk_container_add (GTK_CONTAINER (button), label);
+
+	return button;
+}
+
+/**
+ * gnumeric_dialog_add_button
+ *
+ * Code from gedit
+ *
+ * Creates and adds a button with stock image to the action area of an existing dialog.
+ * 
+ * dialog : dialog you want to add a button
+ * text : button label
+ * sotck_id : stock icon id
+ * response_id : respond id when button clicked
+ *
+ * return : newly created button
+ *
+ **/
+
+GtkWidget*
+gnumeric_dialog_add_button (GtkDialog *dialog, const gchar* text, const gchar* stock_id,
+			    gint response_id)
+{
+	GtkWidget *button;
+
+	g_return_val_if_fail (GTK_IS_DIALOG (dialog), NULL);
+	g_return_val_if_fail (text != NULL, NULL);
+	g_return_val_if_fail (stock_id != NULL, NULL);
+
+	button = gnumeric_button_new_with_stock_image (text, stock_id);
+	g_return_val_if_fail (button != NULL, NULL);
+
+	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+
+	gtk_widget_show (button);
+
+	gtk_dialog_add_action_widget (dialog, button, response_id);	
+
+	return button;
+}
+
+/**
+ * gnumeric_message_dialog_new :
+ *
+ * A convenience fonction to build HIG compliant message dialogs.
+ *
+ *   parent : transient parent, or NULL for none.
+ *   flags 
+ *   type : type of dialog
+ *   primary_message : message displayed in bold
+ *   secondary_message : message displayed below
+ *
+ *   return : a GtkDialog, without buttons.
+ **/
+
+GtkWidget *
+gnumeric_message_dialog_new (GtkWindow * parent,
+			     GtkDialogFlags flags,
+			     GtkMessageType type,
+			     gchar const * primary_message,
+			     gchar const * secondary_message)
+{
+	GtkWidget * dialog;
+	GtkWidget * label;
+	GtkWidget * image;
+	GtkWidget * hbox;
+	gchar * message;
+	const gchar *stock_id = NULL;
+	GtkStockItem item;
+
+	dialog = gtk_dialog_new_with_buttons ("", parent, flags, NULL);
+
+	if (dialog) {
+		image = gtk_image_new (); 
+
+		switch (type) {
+		case GTK_MESSAGE_INFO:
+			stock_id = GTK_STOCK_DIALOG_INFO;
+			break;
+
+		case GTK_MESSAGE_QUESTION:
+			stock_id = GTK_STOCK_DIALOG_QUESTION;
+			break;
+
+		case GTK_MESSAGE_WARNING:
+			stock_id = GTK_STOCK_DIALOG_WARNING;
+			break;
+
+		case GTK_MESSAGE_ERROR:
+			stock_id = GTK_STOCK_DIALOG_ERROR;
+			break;
+
+		default:
+			g_warning ("Unknown GtkMessageType %d", type);
+			break;
+		}
+
+		if (stock_id == NULL)
+			stock_id = GTK_STOCK_DIALOG_INFO;
+
+		if (gtk_stock_lookup (stock_id, &item)) {
+			gtk_image_set_from_stock (GTK_IMAGE (image), stock_id,
+						  GTK_ICON_SIZE_DIALOG);
+
+			gtk_window_set_title (GTK_WINDOW (dialog), item.label);
+		} else
+			g_warning ("Stock dialog ID doesn't exist?");  
+
+		if (primary_message) {
+			if (secondary_message) {
+				message = g_strdup_printf ("<b>%s</b>\n\n%s",
+							   primary_message,
+							   secondary_message);
+			} else {
+				message = g_strdup_printf ("<b>%s</b>",
+							   primary_message);
+			}
+		} else {
+			message = g_strdup_printf (secondary_message);
+		}
+		label = gtk_label_new (message);
+		g_free (message);
+
+		hbox = gtk_hbox_new (FALSE, 0);
+		gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, TRUE, 0);
+		gtk_box_pack_start_defaults (GTK_BOX (hbox),
+					     label);
+		gtk_box_pack_start_defaults (GTK_BOX (GTK_DIALOG (dialog)->vbox),
+					     hbox);
+
+		gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
+		gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+		gtk_misc_set_alignment (GTK_MISC (label), 0.0 , 0.0);
+		gtk_misc_set_alignment (GTK_MISC (label), 0.0 , 0.0);
+		gtk_box_set_spacing (GTK_BOX (hbox), 12);
+		gtk_container_set_border_width (GTK_CONTAINER (hbox), 6);
+		gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 12);
+		gtk_container_set_border_width (GTK_CONTAINER (dialog), 6);
+		gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+		gtk_window_set_resizable (GTK_WINDOW(dialog), FALSE);
+		gtk_widget_show_all (GTK_WIDGET (GTK_DIALOG (dialog)->vbox));
+	}
+
+	return dialog;
+}
