@@ -20,9 +20,26 @@
 #include "format.h"
 #include "sheet-style.h"
 #include "application.h"
+#include "gutils.h"
 
 #include <stdio.h>
 
+#ifndef USE_MSTYLE_POOL
+#define USE_MSTYLE_POOL 1
+#endif
+
+#if USE_MSTYLE_POOL
+/* Memory pool for mstyles.  */
+static gnm_mem_chunk *mstyle_pool;
+#define CHUNK_ALLOC(T,p) ((T*)gnm_mem_chunk_alloc (p))
+#define CHUNK_ALLOC0(T,p) ((T*)gnm_mem_chunk_alloc0 (p))
+#define CHUNK_FREE(p,v) gnm_mem_chunk_free ((p), (v))
+#else
+#define CHUNK_ALLOC(T,c) g_new (T,1)
+#define CHUNK_ALLOC0(T,c) g_new0 (T,1)
+#define CHUNK_FREE(p,v) g_free ((v))
+#endif
+
 typedef struct {
 	MStyleElementType type;
 	union {
@@ -239,15 +256,15 @@ mstyle_element_dump (const MStyleElement *e)
 		break;
 	case MSTYLE_COLOR_BACK:
 		g_string_printf (ans, "\tbackground col %hx:%hx:%hx\n",
-				 e->u.color.any->red,
-				 e->u.color.any->green,
-				 e->u.color.any->blue);
+				 e->u.color.any->color.red,
+				 e->u.color.any->color.green,
+				 e->u.color.any->color.blue);
 		break;
 	case MSTYLE_COLOR_PATTERN:
 		g_string_printf (ans, "\tpattern col %hx:%hx:%hx\n",
-				 e->u.color.any->red,
-				 e->u.color.any->green,
-				 e->u.color.any->blue);
+				 e->u.color.any->color.red,
+				 e->u.color.any->color.green,
+				 e->u.color.any->color.blue);
 		break;
 	case MSTYLE_BORDER_TOP:
 	case MSTYLE_BORDER_BOTTOM:
@@ -267,9 +284,9 @@ mstyle_element_dump (const MStyleElement *e)
 
 	case MSTYLE_COLOR_FORE:
 		g_string_printf (ans, "\tforegnd col %hx:%hx:%hx\n",
-				 e->u.color.any->red,
-				 e->u.color.any->green,
-				 e->u.color.any->blue);
+				 e->u.color.any->color.red,
+				 e->u.color.any->color.green,
+				 e->u.color.any->color.blue);
 		break;
 	case MSTYLE_FONT_NAME:
 		g_string_printf (ans, "\tname '%s'\n", e->u.font.name->str);
@@ -580,7 +597,7 @@ mstyle_font_clear (MStyle *mstyle)
 MStyle *
 mstyle_new (void)
 {
-	MStyle *style = g_new0 (MStyle, 1);
+	MStyle *style = CHUNK_ALLOC0 (MStyle, mstyle_pool);
 
 	style->ref_count = 1;
 	style->link_count = 0;
@@ -595,7 +612,7 @@ mstyle_new (void)
 MStyle *
 mstyle_copy (const MStyle *style)
 {
-	MStyle *new_style = g_new (MStyle, 1);
+	MStyle *new_style = CHUNK_ALLOC (MStyle, mstyle_pool);
 
 	new_style->ref_count = 1;
 	new_style->link_count = 0;
@@ -617,7 +634,7 @@ MStyle *
 mstyle_copy_merge (const MStyle *orig, const MStyle *overlay)
 {
 	int i;
-	MStyle *res = g_new0 (MStyle, 1);
+	MStyle *res = CHUNK_ALLOC0 (MStyle, mstyle_pool);
 
 	MStyleElement       *res_e;
 	const MStyleElement *orig_e;
@@ -742,7 +759,7 @@ mstyle_unref (MStyle *style)
 		mstyle_pango_clear (style);
 		mstyle_font_clear (style);
 
-		g_free (style);
+		CHUNK_FREE (mstyle_pool, style);
 	}
 }
 
@@ -844,11 +861,13 @@ MStyle *
 mstyle_link_sheet (MStyle *style, Sheet *sheet)
 {
 	StyleColor *auto_color;
-	MStyle *orig = style;
+	gboolean style_is_orig = TRUE;
 
 	if (style->linked_sheet != NULL) {
+		MStyle *orig = style;
 		style = mstyle_copy (style);
 		mstyle_unref (orig);
+		style_is_orig = FALSE;
 
 		/* safety test */
 		g_return_val_if_fail (style->linked_sheet != sheet, style);
@@ -859,8 +878,8 @@ mstyle_link_sheet (MStyle *style, Sheet *sheet)
 
 	auto_color = sheet_style_get_auto_pattern_color (sheet);
 	if (mstyle_is_element_set (style, MSTYLE_COLOR_PATTERN))
-		style = link_pattern_color (style, auto_color, style == orig);
-	style = link_border_colors (style, auto_color, style == orig);
+		style = link_pattern_color (style, auto_color, style_is_orig);
+	style = link_border_colors (style, auto_color, style_is_orig);
 	style_color_unref (auto_color);
 
 	style->linked_sheet = sheet;
@@ -1179,7 +1198,7 @@ mstyle_get_pattern (const MStyle *style)
 }
 
 StyleFont *
-mstyle_get_font (const MStyle *style, double zoom)
+mstyle_get_font (const MStyle *style, PangoContext *context, double zoom)
 {
 	g_return_val_if_fail (style != NULL, NULL);
 
@@ -1211,7 +1230,8 @@ mstyle_get_font (const MStyle *style, double zoom)
 			size = DEFAULT_SIZE;
 
 		((MStyle *)style)->font =
-			style_font_new (name, size, zoom, bold, italic);
+			style_font_new (context, name, size,
+					zoom, bold, italic);
 		((MStyle *)style)->font_zoom = zoom;
 	}
 
@@ -1593,7 +1613,9 @@ mstyle_visible_in_blank (const MStyle *st)
 }
 
 PangoAttrList *
-mstyle_get_pango_attrs (const MStyle *mstyle, double zoom)
+mstyle_get_pango_attrs (const MStyle *mstyle,
+			PangoContext *context,
+			double zoom)
 {
 	PangoAttribute *attr;
 	PangoAttrList *res;
@@ -1613,7 +1635,7 @@ mstyle_get_pango_attrs (const MStyle *mstyle, double zoom)
 	/* See http://bugzilla.gnome.org/show_bug.cgi?id=105322 */
 	if (0) {
 		const StyleColor *fore = mstyle_get_color (mstyle, MSTYLE_COLOR_FORE);
-		attr = pango_attr_foreground_new (fore->red, fore->green, fore->blue);
+		attr = pango_attr_foreground_new (fore->color.red, fore->color.green, fore->color.blue);
 		attr->start_index = 0;
 		attr->end_index = -1;
 		pango_attr_list_insert (res, attr);
@@ -1649,7 +1671,7 @@ mstyle_get_pango_attrs (const MStyle *mstyle, double zoom)
 
 	/* Handle font.  */
 	{
-		StyleFont *font = mstyle_get_font (mstyle, zoom);
+		StyleFont *font = mstyle_get_font (mstyle, context, zoom);
 		attr = pango_attr_font_desc_new (font->pango.font_descr);
 		attr->start_index = 0;
 		attr->end_index = -1;
@@ -1660,3 +1682,36 @@ mstyle_get_pango_attrs (const MStyle *mstyle, double zoom)
 	pango_attr_list_ref (res);
 	return res;
 }
+
+/* ------------------------------------------------------------------------- */
+
+void
+mstyle_init (void)
+{
+#if USE_MSTYLE_POOL
+	mstyle_pool =
+		gnm_mem_chunk_new ("mstyle pool",
+				   sizeof (MStyle),
+				   16 * 1024 - 128);
+#endif
+}
+
+#if USE_MSTYLE_POOL
+static void
+cb_mstyle_pool_leak (gpointer data, gpointer user)
+{
+	MStyle *mstyle = data;
+	fprintf (stderr, "Leaking mstyle at %p.\n", mstyle);
+	mstyle_dump (mstyle);
+}
+#endif
+
+void
+mstyle_shutdown (void)
+{
+#if USE_MSTYLE_POOL
+	gnm_mem_chunk_foreach_leak (mstyle_pool, cb_mstyle_pool_leak, NULL);
+	gnm_mem_chunk_destroy (mstyle_pool, FALSE);
+	mstyle_pool = NULL;
+#endif
+}
