@@ -40,9 +40,7 @@ typedef enum _MStyleElementType {
 	        MSTYLE_ALIGN_V,
 	        MSTYLE_ALIGN_H,
 	        MSTYLE_INDENT,
-
-		MSTYLE_ORIENTATION,
-
+		MSTYLE_ROTATION,
 		MSTYLE_WRAP_TEXT,
 		MSTYLE_SHRINK_TO_FIT,
 
@@ -51,7 +49,8 @@ typedef enum _MStyleElementType {
 
 	/* Things not in MS Excel's Style */
 	        MSTYLE_VALIDATION,
-	        MSTYLE_HLINK, /* patch equal_XL if this is changed */
+	        MSTYLE_HLINK,		/* patch equal_XL if this is changed */
+	        MSTYLE_INPUT_MSG,	/* patch equal_XL if this is changed */
 	/* Delimiter */
 	MSTYLE_ELEMENT_MAX
 } MStyleElementType;
@@ -104,7 +103,9 @@ void                mstyle_set_font_size   (MStyle *st, double size);
 double              mstyle_get_font_size   (const MStyle *st);
 
 /* this font must be unrefd after use */
-StyleFont          *mstyle_get_font        (const MStyle *st, double zoom);
+StyleFont          *mstyle_get_font        (const MStyle *st,
+					    PangoContext *context,
+					    double zoom);
 void                mstyle_set_format      (MStyle *st, StyleFormat *);
 void                mstyle_set_format_text (MStyle *st, const char *format);
 StyleFormat        *mstyle_get_format      (const MStyle *st);
@@ -114,10 +115,13 @@ void                mstyle_set_align_v     (MStyle *st, StyleVAlignFlags a);
 StyleVAlignFlags    mstyle_get_align_v     (const MStyle *st);
 void                mstyle_set_indent	   (MStyle *st, int i);
 int		    mstyle_get_indent	   (const MStyle *st);
-void                mstyle_set_orientation (MStyle *st, StyleOrientation o);
-StyleOrientation    mstyle_get_orientation (const MStyle *st);
+
+void                mstyle_set_rotation	   (MStyle *st, int r);
+int            	    mstyle_get_rotation    (const MStyle *st);
+
 void                mstyle_set_wrap_text   (MStyle *st, gboolean f);
 gboolean            mstyle_get_wrap_text   (const MStyle *st);
+gboolean            mstyle_get_effective_wrap_text   (const MStyle *st);
 void                mstyle_set_shrink_to_fit (MStyle *st, gboolean f);
 gboolean            mstyle_get_shrink_to_fit (const MStyle *st);
 
@@ -126,15 +130,25 @@ gboolean            mstyle_get_content_locked (const MStyle *st);
 void                mstyle_set_content_hidden (MStyle *st, gboolean f);
 gboolean            mstyle_get_content_hidden (const MStyle *st);
 
-void                mstyle_set_validation       (MStyle *st, Validation *v);
-Validation         *mstyle_get_validation       (const MStyle *st);
+void                mstyle_set_validation	(MStyle *st, Validation *v);
+Validation         *mstyle_get_validation	(const MStyle *st);
 
 void                mstyle_set_hlink		(MStyle *st, GnmHLink *link);
 GnmHLink	   *mstyle_get_hlink		(const MStyle *st);
 
-gboolean            mstyle_visible_in_blank(const MStyle *st);
+void                mstyle_set_input_msg	(MStyle *st, GnmInputMsg *msg);
+GnmInputMsg   	   *mstyle_get_input_msg	(const MStyle *st);
+
+gboolean            mstyle_visible_in_blank (const MStyle *st);
+
+PangoAttrList      *mstyle_get_pango_attrs (const MStyle *st,
+					    PangoContext *context,
+					    double zoom);
 
 char       *mstyle_to_string   (const MStyle *st); /* Debug only ! leaks like a sieve */
 void        mstyle_dump        (const MStyle *st);
 
+void        mstyle_init (void);
+void        mstyle_shutdown (void);
+
 #endif /* GNUMERIC_MSTYLE_H */
