@@ -1,7 +1,7 @@
-/* A Bison parser, made from parser.y, by GNU bison 1.75.  */
+/* A Bison parser, made by GNU Bison 1.875a.  */
 
 /* Skeleton parser for Yacc-like parsing with Bison,
-   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002 Free Software Foundation, Inc.
+   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -34,10 +34,13 @@
    USER NAME SPACE" below.  */
 
 /* Identify Bison output.  */
-#define YYBISON	1
+#define YYBISON 1
+
+/* Skeleton name.  */
+#define YYSKELETON_NAME "yacc.c"
 
 /* Pure parsers.  */
-#define YYPURE	0
+#define YYPURE 0
 
 /* Using locations.  */
 #define YYLSP_NEEDED 0
@@ -574,39 +577,30 @@ int yyparse (void);
 # define YYERROR_VERBOSE 0
 #endif
 
-#ifndef YYSTYPE
+#if ! defined (YYSTYPE) && ! defined (YYSTYPE_IS_DECLARED)
 #line 470 "parser.y"
-typedef union {
+typedef union YYSTYPE {
 	GnmExpr		*expr;
 	Value		*value;
 	CellRef		*cell;
 	GnmExprList	*list;
 	Sheet		*sheet;
 	Workbook	*wb;
-} yystype;
-/* Line 193 of /usr/share/bison/yacc.c.  */
-#line 589 "y.tab.c"
-# define YYSTYPE yystype
+} YYSTYPE;
+/* Line 191 of yacc.c.  */
+#line 592 "y.tab.c"
+# define yystype YYSTYPE /* obsolescent; will be withdrawn */
+# define YYSTYPE_IS_DECLARED 1
 # define YYSTYPE_IS_TRIVIAL 1
 #endif
 
-#ifndef YYLTYPE
-typedef struct yyltype
-{
-  int first_line;
-  int first_column;
-  int last_line;
-  int last_column;
-} yyltype;
-# define YYLTYPE yyltype
-# define YYLTYPE_IS_TRIVIAL 1
-#endif
+
 
 /* Copy the second part of user declarations.  */
 
 
-/* Line 213 of /usr/share/bison/yacc.c.  */
-#line 610 "y.tab.c"
+/* Line 214 of yacc.c.  */
+#line 604 "y.tab.c"
 
 #if ! defined (yyoverflow) || YYERROR_VERBOSE
 
@@ -642,7 +636,7 @@ typedef struct yyltype
 
 #if (! defined (yyoverflow) \
      && (! defined (__cplusplus) \
-	 || (YYLTYPE_IS_TRIVIAL && YYSTYPE_IS_TRIVIAL)))
+	 || (YYSTYPE_IS_TRIVIAL)))
 
 /* A type that is properly aligned for any stack member.  */
 union yyalloc
@@ -652,13 +646,13 @@ union yyalloc
   };
 
 /* The size of the maximum gap between one aligned stack and the next.  */
-# define YYSTACK_GAP_MAX (sizeof (union yyalloc) - 1)
+# define YYSTACK_GAP_MAXIMUM (sizeof (union yyalloc) - 1)
 
 /* The size of an array large to enough to hold all stacks, each with
    N elements.  */
 # define YYSTACK_BYTES(N) \
      ((N) * (sizeof (short) + sizeof (YYSTYPE))				\
-      + YYSTACK_GAP_MAX)
+      + YYSTACK_GAP_MAXIMUM)
 
 /* Copy COUNT objects from FROM to TO.  The source and destination do
    not overlap.  */
@@ -672,7 +666,7 @@ union yyalloc
 	{					\
 	  register YYSIZE_T yyi;		\
 	  for (yyi = 0; yyi < (Count); yyi++)	\
-	    (To)[yyi] = (From)[yyi];	\
+	    (To)[yyi] = (From)[yyi];		\
 	}					\
       while (0)
 #  endif
@@ -689,7 +683,7 @@ union yyalloc
 	YYSIZE_T yynewbytes;						\
 	YYCOPY (&yyptr->Stack, Stack, yysize);				\
 	Stack = &yyptr->Stack;						\
-	yynewbytes = yystacksize * sizeof (*Stack) + YYSTACK_GAP_MAX;	\
+	yynewbytes = yystacksize * sizeof (*Stack) + YYSTACK_GAP_MAXIMUM; \
 	yyptr += yynewbytes / sizeof (*yyptr);				\
       }									\
     while (0)
@@ -704,6 +698,7 @@ union yyalloc
 
 /* YYFINAL -- State number of the termination state. */
 #define YYFINAL  4
+/* YYLAST -- Last index in YYTABLE.  */
 #define YYLAST   197
 
 /* YYNTOKENS -- Number of terminals. */
@@ -719,8 +714,8 @@ union yyalloc
 #define YYUNDEFTOK  2
 #define YYMAXUTOK   274
 
-#define YYTRANSLATE(X) \
-  ((unsigned)(X) <= YYMAXUTOK ? yytranslate[X] : YYUNDEFTOK)
+#define YYTRANSLATE(YYX) 						\
+  ((unsigned int) (YYX) <= YYMAXUTOK ? yytranslate[YYX] : YYUNDEFTOK)
 
 /* YYTRANSLATE[YYLEX] -- Bison symbol number corresponding to YYLEX.  */
 static const unsigned char yytranslate[] =
@@ -797,9 +792,9 @@ static const unsigned short yyrline[] =
        0,   498,   498,   504,   512,   518,   521,   522,   523,   524,
      525,   526,   527,   528,   529,   530,   531,   532,   533,   534,
      535,   536,   537,   538,   539,   544,   548,   549,   550,   552,
-     572,   578,   579,   597,   618,   645,   646,   649,   672,   681,
-     693,   694,   695,   699,   705,   710,   716,   726,   729,   730,
-     731,   734,   745,   758,   771,   774,   779
+     572,   578,   579,   597,   618,   650,   651,   654,   677,   686,
+     698,   699,   700,   704,   710,   715,   721,   731,   734,   735,
+     736,   739,   750,   763,   776,   779,   784
 };
 #endif
 
@@ -904,7 +899,7 @@ static const yysigned_char yypgoto[] =
 /* YYTABLE[YYPACT[STATE-NUM]].  What to do in state STATE-NUM.  If
    positive, shift that token.  If negative, reduce the rule which
    number is the opposite.  If zero, do what YYDEFACT says.
-   If YYTABLE_NINF, parse error.  */
+   If YYTABLE_NINF, syntax error.  */
 #define YYTABLE_NINF -37
 static const yysigned_char yytable[] =
 {
@@ -988,13 +983,14 @@ static const unsigned char yystos[] =
 
 #define yyerrok		(yyerrstatus = 0)
 #define yyclearin	(yychar = YYEMPTY)
-#define YYEMPTY		-2
+#define YYEMPTY		(-2)
 #define YYEOF		0
 
 #define YYACCEPT	goto yyacceptlab
 #define YYABORT		goto yyabortlab
 #define YYERROR		goto yyerrlab1
 
+
 /* Like YYERROR except do call yyerror.  This remains here temporarily
    to ease the transition to the new meaning of YYERROR, for GCC.
    Once GCC version 2 has supplanted version 1, this can go.  */
@@ -1009,13 +1005,13 @@ do								\
     {								\
       yychar = (Token);						\
       yylval = (Value);						\
-      yychar1 = YYTRANSLATE (yychar);				\
+      yytoken = YYTRANSLATE (yychar);				\
       YYPOPSTACK;						\
       goto yybackup;						\
     }								\
   else								\
     { 								\
-      yyerror ("syntax error: cannot back up");			\
+      yyerror ("syntax error: cannot back up");\
       YYERROR;							\
     }								\
 while (0)
@@ -1027,7 +1023,7 @@ while (0)
    are run).  */
 
 #ifndef YYLLOC_DEFAULT
-# define YYLLOC_DEFAULT(Current, Rhs, N)           \
+# define YYLLOC_DEFAULT(Current, Rhs, N)         \
   Current.first_line   = Rhs[1].first_line;      \
   Current.first_column = Rhs[1].first_column;    \
   Current.last_line    = Rhs[N].last_line;       \
@@ -1036,7 +1032,11 @@ while (0)
 
 /* YYLEX -- calling `yylex' with the right arguments.  */
 
-#define YYLEX	yylex ()
+#ifdef YYLEX_PARAM
+# define YYLEX yylex (YYLEX_PARAM)
+#else
+# define YYLEX yylex ()
+#endif
 
 /* Enable debugging if requested.  */
 #if YYDEBUG
@@ -1051,19 +1051,93 @@ do {						\
   if (yydebug)					\
     YYFPRINTF Args;				\
 } while (0)
+
 # define YYDSYMPRINT(Args)			\
 do {						\
   if (yydebug)					\
     yysymprint Args;				\
 } while (0)
+
+# define YYDSYMPRINTF(Title, Token, Value, Location)		\
+do {								\
+  if (yydebug)							\
+    {								\
+      YYFPRINTF (stderr, "%s ", Title);				\
+      yysymprint (stderr, 					\
+                  Token, Value);	\
+      YYFPRINTF (stderr, "\n");					\
+    }								\
+} while (0)
+
+/*------------------------------------------------------------------.
+| yy_stack_print -- Print the state stack from its BOTTOM up to its |
+| TOP (cinluded).                                                   |
+`------------------------------------------------------------------*/
+
+#if defined (__STDC__) || defined (__cplusplus)
+static void
+yy_stack_print (short *bottom, short *top)
+#else
+static void
+yy_stack_print (bottom, top)
+    short *bottom;
+    short *top;
+#endif
+{
+  YYFPRINTF (stderr, "Stack now");
+  for (/* Nothing. */; bottom <= top; ++bottom)
+    YYFPRINTF (stderr, " %d", *bottom);
+  YYFPRINTF (stderr, "\n");
+}
+
+# define YY_STACK_PRINT(Bottom, Top)				\
+do {								\
+  if (yydebug)							\
+    yy_stack_print ((Bottom), (Top));				\
+} while (0)
+
+
+/*------------------------------------------------.
+| Report that the YYRULE is going to be reduced.  |
+`------------------------------------------------*/
+
+#if defined (__STDC__) || defined (__cplusplus)
+static void
+yy_reduce_print (int yyrule)
+#else
+static void
+yy_reduce_print (yyrule)
+    int yyrule;
+#endif
+{
+  int yyi;
+  unsigned int yylineno = yyrline[yyrule];
+  YYFPRINTF (stderr, "Reducing stack by rule %d (line %u), ",
+             yyrule - 1, yylineno);
+  /* Print the symbols being reduced, and their result.  */
+  for (yyi = yyprhs[yyrule]; 0 <= yyrhs[yyi]; yyi++)
+    YYFPRINTF (stderr, "%s ", yytname [yyrhs[yyi]]);
+  YYFPRINTF (stderr, "-> %s\n", yytname [yyr1[yyrule]]);
+}
+
+# define YY_REDUCE_PRINT(Rule)		\
+do {					\
+  if (yydebug)				\
+    yy_reduce_print (Rule);		\
+} while (0)
+
 /* Nonzero means print parse trace.  It is left uninitialized so that
    multiple parsers can coexist.  */
 int yydebug;
 #else /* !YYDEBUG */
 # define YYDPRINTF(Args)
 # define YYDSYMPRINT(Args)
+# define YYDSYMPRINTF(Title, Token, Value, Location)
+# define YY_STACK_PRINT(Bottom, Top)
+# define YY_REDUCE_PRINT(Rule)
 #endif /* !YYDEBUG */
 
+
 /* YYINITDEPTH -- initial size of the parser's stacks.  */
 #ifndef	YYINITDEPTH
 # define YYINITDEPTH 200
@@ -1142,95 +1216,85 @@ yystpcpy (yydest, yysrc)
 
 
 #if YYDEBUG
-/*-----------------------------.
-| Print this symbol on YYOUT.  |
-`-----------------------------*/
+/*--------------------------------.
+| Print this symbol on YYOUTPUT.  |
+`--------------------------------*/
 
-static void
 #if defined (__STDC__) || defined (__cplusplus)
-yysymprint (FILE* yyout, int yytype, YYSTYPE yyvalue)
+static void
+yysymprint (FILE *yyoutput, int yytype, YYSTYPE *yyvaluep)
 #else
-yysymprint (yyout, yytype, yyvalue)
-    FILE* yyout;
+static void
+yysymprint (yyoutput, yytype, yyvaluep)
+    FILE *yyoutput;
     int yytype;
-    YYSTYPE yyvalue;
+    YYSTYPE *yyvaluep;
 #endif
 {
   /* Pacify ``unused variable'' warnings.  */
-  (void) yyvalue;
+  (void) yyvaluep;
 
   if (yytype < YYNTOKENS)
     {
-      YYFPRINTF (yyout, "token %s (", yytname[yytype]);
+      YYFPRINTF (yyoutput, "token %s (", yytname[yytype]);
 # ifdef YYPRINT
-      YYPRINT (yyout, yytoknum[yytype], yyvalue);
+      YYPRINT (yyoutput, yytoknum[yytype], *yyvaluep);
 # endif
     }
   else
-    YYFPRINTF (yyout, "nterm %s (", yytname[yytype]);
+    YYFPRINTF (yyoutput, "nterm %s (", yytname[yytype]);
 
   switch (yytype)
     {
       default:
         break;
     }
-  YYFPRINTF (yyout, ")");
+  YYFPRINTF (yyoutput, ")");
 }
-#endif /* YYDEBUG. */
-
 
+#endif /* ! YYDEBUG */
 /*-----------------------------------------------.
 | Release the memory associated to this symbol.  |
 `-----------------------------------------------*/
 
-static void
 #if defined (__STDC__) || defined (__cplusplus)
-yydestruct (int yytype, YYSTYPE yyvalue)
+static void
+yydestruct (int yytype, YYSTYPE *yyvaluep)
 #else
-yydestruct (yytype, yyvalue)
+static void
+yydestruct (yytype, yyvaluep)
     int yytype;
-    YYSTYPE yyvalue;
+    YYSTYPE *yyvaluep;
 #endif
 {
   /* Pacify ``unused variable'' warnings.  */
-  (void) yyvalue;
+  (void) yyvaluep;
 
   switch (yytype)
     {
+
       default:
         break;
     }
 }
-
 
 
-/* The user can define YYPARSE_PARAM as the name of an argument to be passed
-   into yyparse.  The argument should have type void *.
-   It should actually point to an object.
-   Grammar actions can access the variable by casting it
-   to the proper pointer type.  */
+/* Prevent warnings from -Wmissing-prototypes.  */
 
 #ifdef YYPARSE_PARAM
 # if defined (__STDC__) || defined (__cplusplus)
-#  define YYPARSE_PARAM_ARG void *YYPARSE_PARAM
-#  define YYPARSE_PARAM_DECL
+int yyparse (void *YYPARSE_PARAM);
 # else
-#  define YYPARSE_PARAM_ARG YYPARSE_PARAM
-#  define YYPARSE_PARAM_DECL void *YYPARSE_PARAM;
+int yyparse ();
 # endif
-#else /* !YYPARSE_PARAM */
-# define YYPARSE_PARAM_ARG
-# define YYPARSE_PARAM_DECL
-#endif /* !YYPARSE_PARAM */
-
-/* Prevent warning if -Wstrict-prototypes.  */
-#ifdef __GNUC__
-# ifdef YYPARSE_PARAM
-int yyparse (void *);
-# else
+#else /* ! YYPARSE_PARAM */
+#if defined (__STDC__) || defined (__cplusplus)
 int yyparse (void);
-# endif
+#else
+int yyparse ();
 #endif
+#endif /* ! YYPARSE_PARAM */
+
 
 
 /* The lookahead symbol.  */
@@ -1239,13 +1303,32 @@ int yychar;
 /* The semantic value of the lookahead symbol.  */
 YYSTYPE yylval;
 
-/* Number of parse errors so far.  */
+/* Number of syntax errors so far.  */
 int yynerrs;
 
 
+
+/*----------.
+| yyparse.  |
+`----------*/
+
+#ifdef YYPARSE_PARAM
+# if defined (__STDC__) || defined (__cplusplus)
+int yyparse (void *YYPARSE_PARAM)
+# else
+int yyparse (YYPARSE_PARAM)
+  void *YYPARSE_PARAM;
+# endif
+#else /* ! YYPARSE_PARAM */
+#if defined (__STDC__) || defined (__cplusplus)
 int
-yyparse (YYPARSE_PARAM_ARG)
-     YYPARSE_PARAM_DECL
+yyparse (void)
+#else
+int
+yyparse ()
+
+#endif
+#endif
 {
   
   register int yystate;
@@ -1254,7 +1337,7 @@ yyparse (YYPARSE_PARAM_ARG)
   /* Number of tokens to shift before error messages enabled.  */
   int yyerrstatus;
   /* Lookahead token as an internal (translated) token number.  */
-  int yychar1 = 0;
+  int yytoken = 0;
 
   /* Three stacks and their tools:
      `yyss': related to states,
@@ -1318,7 +1401,7 @@ yyparse (YYPARSE_PARAM_ARG)
  yysetstate:
   *yyssp = yystate;
 
-  if (yyssp >= yyss + yystacksize - 1)
+  if (yyss + yystacksize - 1 <= yyssp)
     {
       /* Get the current used size of the three stacks, in elements.  */
       YYSIZE_T yysize = yyssp - yyss + 1;
@@ -1350,10 +1433,10 @@ yyparse (YYPARSE_PARAM_ARG)
       goto yyoverflowlab;
 # else
       /* Extend the stack our own way.  */
-      if (yystacksize >= YYMAXDEPTH)
+      if (YYMAXDEPTH <= yystacksize)
 	goto yyoverflowlab;
       yystacksize *= 2;
-      if (yystacksize > YYMAXDEPTH)
+      if (YYMAXDEPTH < yystacksize)
 	yystacksize = YYMAXDEPTH;
 
       {
@@ -1379,7 +1462,7 @@ yyparse (YYPARSE_PARAM_ARG)
       YYDPRINTF ((stderr, "Stack size increased to %lu\n",
 		  (unsigned long int) yystacksize));
 
-      if (yyssp >= yyss + yystacksize - 1)
+      if (yyss + yystacksize - 1 <= yyssp)
 	YYABORT;
     }
 
@@ -1404,39 +1487,28 @@ yybackup:
 
   /* Not known => get a lookahead token if don't already have one.  */
 
-  /* yychar is either YYEMPTY or YYEOF
-     or a valid token in external form.  */
-
+  /* YYCHAR is either YYEMPTY or YYEOF or a valid lookahead symbol.  */
   if (yychar == YYEMPTY)
     {
       YYDPRINTF ((stderr, "Reading a token: "));
       yychar = YYLEX;
     }
 
-  /* Convert token to internal form (in yychar1) for indexing tables with.  */
-
-  if (yychar <= 0)		/* This means end of input.  */
+  if (yychar <= YYEOF)
     {
-      yychar1 = 0;
-      yychar = YYEOF;		/* Don't call YYLEX any more.  */
-
+      yychar = yytoken = YYEOF;
       YYDPRINTF ((stderr, "Now at end of input.\n"));
     }
   else
     {
-      yychar1 = YYTRANSLATE (yychar);
-
-      /* We have to keep this `#if YYDEBUG', since we use variables
-	 which are defined only if `YYDEBUG' is set.  */
-      YYDPRINTF ((stderr, "Next token is "));
-      YYDSYMPRINT ((stderr, yychar1, yylval));
-      YYDPRINTF ((stderr, "\n"));
+      yytoken = YYTRANSLATE (yychar);
+      YYDSYMPRINTF ("Next token is", yytoken, &yylval, &yylloc);
     }
 
-  /* If the proper action on seeing token YYCHAR1 is to reduce or to
+  /* If the proper action on seeing token YYTOKEN is to reduce or to
      detect an error, take that action.  */
-  yyn += yychar1;
-  if (yyn < 0 || YYLAST < yyn || yycheck[yyn] != yychar1)
+  yyn += yytoken;
+  if (yyn < 0 || YYLAST < yyn || yycheck[yyn] != yytoken)
     goto yydefault;
   yyn = yytable[yyn];
   if (yyn <= 0)
@@ -1451,8 +1523,7 @@ yybackup:
     YYACCEPT;
 
   /* Shift the lookahead token.  */
-  YYDPRINTF ((stderr, "Shifting token %d (%s), ",
-	      yychar, yytname[yychar1]));
+  YYDPRINTF ((stderr, "Shifting token %s, ", yytname[yytoken]));
 
   /* Discard the token being shifted unless it is eof.  */
   if (yychar != YYEOF)
@@ -1498,23 +1569,7 @@ yyreduce:
   yyval = yyvsp[1-yylen];
 
 
-
-#if YYDEBUG
-  /* We have to keep this `#if YYDEBUG', since we use variables which
-     are defined only if `YYDEBUG' is set.  */
-  if (yydebug)
-    {
-      int yyi;
-
-      YYFPRINTF (stderr, "Reducing via rule %d (line %d), ",
-		 yyn - 1, yyrline[yyn]);
-
-      /* Print the symbols being reduced, and their result.  */
-      for (yyi = yyprhs[yyn]; yyrhs[yyi] >= 0; yyi++)
-	YYFPRINTF (stderr, "%s ", yytname[yyrhs[yyi]]);
-      YYFPRINTF (stderr, " -> %s\n", yytname[yyr1[yyn]]);
-    }
-#endif
+  YY_REDUCE_PRINT (yyn);
   switch (yyn)
     {
         case 2:
@@ -1754,24 +1809,29 @@ yyreduce:
 #line 618 "parser.y"
     {
 		char const *name = yyvsp[-3].expr->constant.value->v_str.val->str;
-		GnmFunc *f = gnm_func_lookup (name, state->pos->wb);
+		GnmFunc *f;
 		GnmExpr const *f_call = NULL;
+		GnmParseFunctionHandler h = NULL;
 
 		yyval.expr = NULL;
 
-		if (f == NULL) {
-			GnmParseFunctionHandler h =
-				state->convs->unknown_function_handler;
-			if (h)
-				f_call = h (name, yyvsp[-1].list, NULL);
-		} else
-			f_call = gnm_expr_new_funcall (f, yyvsp[-1].list);
+		if (state->convs->function_rewriter_hash)
+			h = g_hash_table_lookup (state->convs->function_rewriter_hash, name);
 
-		/* We're done with the function name.  */
-		unregister_allocation (yyvsp[-3].expr); gnm_expr_unref (yyvsp[-3].expr);
-		unregister_allocation (yyvsp[-1].list);
+		if (h) {
+			f_call = h (name, yyvsp[-1].list, state->convs);
+		} else if ((f = gnm_func_lookup (name, state->pos->wb))) {
+			f_call = gnm_expr_new_funcall (f, yyvsp[-1].list);
+		} else {
+			h = state->convs->unknown_function_handler;
+			if (h)
+				f_call = h (name, yyvsp[-1].list, state->convs);
+		}
 
 		if (f_call) {
+			/* We're done with the function name.  */
+			unregister_allocation (yyvsp[-3].expr); gnm_expr_unref (yyvsp[-3].expr);
+			unregister_allocation (yyvsp[-1].list);
 			yyval.expr = register_expr_allocation (f_call);
 		} else {
 			YYERROR;
@@ -1780,7 +1840,7 @@ yyreduce:
     break;
 
   case 37:
-#line 649 "parser.y"
+#line 654 "parser.y"
     {
 		char const *wb_name = yyvsp[-1].expr->constant.value->v_str.val->str;
 		Workbook *wb = application_workbook_get_by_name (wb_name);
@@ -1790,7 +1850,7 @@ yyreduce:
 			yyval.wb = wb;
 		} else {
 			/* kludge to produce better error messages
-			 * we know that the last token read will be the ''
+			 * we know that the last token read will be the ']'
 			 * so subtract 1.
 			 */
 			report_err (state, g_error_new (1, PERR_UNKNOWN_WORKBOOK,
@@ -1802,7 +1862,7 @@ yyreduce:
     break;
 
   case 38:
-#line 672 "parser.y"
+#line 677 "parser.y"
     {
 		Sheet *sheet = parser_sheet_by_name (state->pos->wb, yyvsp[-1].expr);
 		if (sheet != NULL) {
@@ -1815,7 +1875,7 @@ yyreduce:
     break;
 
   case 39:
-#line 681 "parser.y"
+#line 686 "parser.y"
     {
 		Sheet *sheet = parser_sheet_by_name (yyvsp[-2].wb, yyvsp[-1].expr);
 
@@ -1829,17 +1889,17 @@ yyreduce:
     break;
 
   case 40:
-#line 693 "parser.y"
+#line 698 "parser.y"
     { yyval.expr = yyvsp[0].expr; }
     break;
 
   case 41:
-#line 694 "parser.y"
+#line 699 "parser.y"
     { yyval.expr = build_range_ctor (yyvsp[-2].expr, yyvsp[0].expr, NULL); }
     break;
 
   case 42:
-#line 695 "parser.y"
+#line 700 "parser.y"
     {
 		yyval.expr = build_range_ctor (yyvsp[-2].expr, yyvsp[0].expr, yyvsp[-2].expr);
 		if (yyval.expr == NULL) { YYERROR; }
@@ -1847,7 +1907,7 @@ yyreduce:
     break;
 
   case 43:
-#line 699 "parser.y"
+#line 704 "parser.y"
     {
 		yyval.expr = build_range_ctor (yyvsp[-2].expr, yyvsp[0].expr, yyvsp[0].expr);
 		if (yyval.expr == NULL) { YYERROR; }
@@ -1855,7 +1915,7 @@ yyreduce:
     break;
 
   case 44:
-#line 705 "parser.y"
+#line 710 "parser.y"
     {
 		unregister_allocation (yyvsp[0].expr);
 		yyval.list = g_slist_prepend (NULL, yyvsp[0].expr);
@@ -1864,7 +1924,7 @@ yyreduce:
     break;
 
   case 45:
-#line 710 "parser.y"
+#line 715 "parser.y"
     {
 		unregister_allocation (yyvsp[0].list);
 		unregister_allocation (yyvsp[-2].expr);
@@ -1874,7 +1934,7 @@ yyreduce:
     break;
 
   case 46:
-#line 716 "parser.y"
+#line 721 "parser.y"
     {
 		GSList *tmp = yyvsp[0].list;
 		unregister_allocation (yyvsp[0].list);
@@ -1888,27 +1948,27 @@ yyreduce:
     break;
 
   case 47:
-#line 726 "parser.y"
+#line 731 "parser.y"
     { yyval.list = NULL; }
     break;
 
   case 48:
-#line 729 "parser.y"
+#line 734 "parser.y"
     { yyval.expr = yyvsp[0].expr; }
     break;
 
   case 49:
-#line 730 "parser.y"
+#line 735 "parser.y"
     { yyval.expr = fold_negative (yyvsp[0].expr); }
     break;
 
   case 50:
-#line 731 "parser.y"
+#line 736 "parser.y"
     { yyval.expr = parse_string_as_value (yyvsp[0].expr); }
     break;
 
   case 51:
-#line 734 "parser.y"
+#line 739 "parser.y"
     {
 		unregister_allocation (yyvsp[0].expr);
 		yyval.list = g_slist_prepend (NULL, yyvsp[0].expr);
@@ -1917,7 +1977,7 @@ yyreduce:
     break;
 
   case 52:
-#line 745 "parser.y"
+#line 750 "parser.y"
     {
 		if (state->array_col_separator == ',') {
 			unregister_allocation (yyvsp[0].list);
@@ -1934,7 +1994,7 @@ yyreduce:
     break;
 
   case 53:
-#line 758 "parser.y"
+#line 763 "parser.y"
     {
 		if (state->array_col_separator == '\\') {
 			unregister_allocation (yyvsp[0].list);
@@ -1951,12 +2011,12 @@ yyreduce:
     break;
 
   case 54:
-#line 771 "parser.y"
+#line 776 "parser.y"
     { yyval.list = NULL; }
     break;
 
   case 55:
-#line 774 "parser.y"
+#line 779 "parser.y"
     {
 		unregister_allocation (yyvsp[0].list);
 		yyval.list = g_slist_prepend (NULL, yyvsp[0].list);
@@ -1965,7 +2025,7 @@ yyreduce:
     break;
 
   case 56:
-#line 779 "parser.y"
+#line 784 "parser.y"
     {
 		unregister_allocation (yyvsp[0].list);
 		unregister_allocation (yyvsp[-2].list);
@@ -1977,23 +2037,14 @@ yyreduce:
 
     }
 
-/* Line 1016 of /usr/share/bison/yacc.c.  */
-#line 1982 "y.tab.c"
+/* Line 999 of yacc.c.  */
+#line 2042 "y.tab.c"
 
   yyvsp -= yylen;
   yyssp -= yylen;
 
 
-#if YYDEBUG
-  if (yydebug)
-    {
-      short *yyssp1 = yyss - 1;
-      YYFPRINTF (stderr, "state stack now");
-      while (yyssp1 != yyssp)
-	YYFPRINTF (stderr, " %d", *++yyssp1);
-      YYFPRINTF (stderr, "\n");
-    }
-#endif
+  YY_STACK_PRINT (yyss, yyssp);
 
   *++yyvsp = yyval;
 
@@ -2038,12 +2089,12 @@ yyerrlab:
 	       yyx < (int) (sizeof (yytname) / sizeof (char *)); yyx++)
 	    if (yycheck[yyx + yyn] == yyx && yyx != YYTERROR)
 	      yysize += yystrlen (yytname[yyx]) + 15, yycount++;
-	  yysize += yystrlen ("parse error, unexpected ") + 1;
+	  yysize += yystrlen ("syntax error, unexpected ") + 1;
 	  yysize += yystrlen (yytname[yytype]);
 	  yymsg = (char *) YYSTACK_ALLOC (yysize);
 	  if (yymsg != 0)
 	    {
-	      char *yyp = yystpcpy (yymsg, "parse error, unexpected ");
+	      char *yyp = yystpcpy (yymsg, "syntax error, unexpected ");
 	      yyp = yystpcpy (yyp, yytname[yytype]);
 
 	      if (yycount < 5)
@@ -2064,19 +2115,15 @@ yyerrlab:
 	      YYSTACK_FREE (yymsg);
 	    }
 	  else
-	    yyerror ("parse error; also virtual memory exhausted");
+	    yyerror ("syntax error; also virtual memory exhausted");
 	}
       else
 #endif /* YYERROR_VERBOSE */
-	yyerror ("parse error");
+	yyerror ("syntax error");
     }
-  goto yyerrlab1;
 
 
-/*----------------------------------------------------.
-| yyerrlab1 -- error raised explicitly by an action.  |
-`----------------------------------------------------*/
-yyerrlab1:
+
   if (yyerrstatus == 3)
     {
       /* If just tried and failed to reuse lookahead token after an
@@ -2088,28 +2135,30 @@ yyerrlab1:
 	  /* Pop the error token.  */
           YYPOPSTACK;
 	  /* Pop the rest of the stack.  */
-	  while (yyssp > yyss)
+	  while (yyss < yyssp)
 	    {
-	      YYDPRINTF ((stderr, "Error: popping "));
-	      YYDSYMPRINT ((stderr,
-			    yystos[*yyssp],
-			    *yyvsp));
-	      YYDPRINTF ((stderr, "\n"));
-	      yydestruct (yystos[*yyssp], *yyvsp);
+	      YYDSYMPRINTF ("Error: popping", yystos[*yyssp], yyvsp, yylsp);
+	      yydestruct (yystos[*yyssp], yyvsp);
 	      YYPOPSTACK;
 	    }
 	  YYABORT;
         }
 
-      YYDPRINTF ((stderr, "Discarding token %d (%s).\n",
-		  yychar, yytname[yychar1]));
-      yydestruct (yychar1, yylval);
+      YYDSYMPRINTF ("Error: discarding", yytoken, &yylval, &yylloc);
+      yydestruct (yytoken, &yylval);
       yychar = YYEMPTY;
+
     }
 
   /* Else will try to reuse lookahead token after shifting the error
      token.  */
+  goto yyerrlab1;
 
+
+/*----------------------------------------------------.
+| yyerrlab1 -- error raised explicitly by an action.  |
+`----------------------------------------------------*/
+yyerrlab1:
   yyerrstatus = 3;	/* Each real token shifted decrements this.  */
 
   for (;;)
@@ -2130,26 +2179,12 @@ yyerrlab1:
       if (yyssp == yyss)
 	YYABORT;
 
-      YYDPRINTF ((stderr, "Error: popping "));
-      YYDSYMPRINT ((stderr,
-		    yystos[*yyssp], *yyvsp));
-      YYDPRINTF ((stderr, "\n"));
-
-      yydestruct (yystos[yystate], *yyvsp);
+      YYDSYMPRINTF ("Error: popping", yystos[*yyssp], yyvsp, yylsp);
+      yydestruct (yystos[yystate], yyvsp);
       yyvsp--;
       yystate = *--yyssp;
 
-
-#if YYDEBUG
-      if (yydebug)
-	{
-	  short *yyssp1 = yyss - 1;
-	  YYFPRINTF (stderr, "Error: state stack now");
-	  while (yyssp1 != yyssp)
-	    YYFPRINTF (stderr, " %d", *++yyssp1);
-	  YYFPRINTF (stderr, "\n");
-	}
-#endif
+      YY_STACK_PRINT (yyss, yyssp);
     }
 
   if (yyn == YYFINAL)
@@ -2197,13 +2232,13 @@ yyreturn:
 }
 
 
-#line 786 "parser.y"
+#line 791 "parser.y"
 
 
 static char const *
 find_matching_close (char const *str, char const **res)
 {
-	for (; *str; str = g_utf8_next_char (str)) {
+	while (*str) {
 		if (*str == '(') {
 			char const *tmp = str;
 			str = find_matching_close (str + 1, res);
@@ -2215,18 +2250,30 @@ find_matching_close (char const *str, char const **res)
 			return str;
 		else if (*str == '\'' || *str == '\"') {
 			GString *dummy = g_string_new ("");
-			const char *end = gnm_strunescape (dummy, str);
+			char const *end = gnm_strunescape (dummy, str);
 			g_string_free (dummy, TRUE);
-			if (end)
-				str = end;
-			else
+			if (end == NULL)
 				return str + strlen (str);
+			str = end;
+			continue; /* skip incrementing str */
 		}
+		str = g_utf8_next_char (str);
 	}
 
 	return str;
 }
 
+static inline int
+eat_space (ParserState *state, int res)
+{
+	/* help the user by ignoring pointless spaces after a
+	 * seperator.  We know they are going to be errors and
+	 * the spaces can not be operators in this context */
+	while (*state->ptr == ' ')
+		state->ptr++;
+	return res;
+}
+
 static int
 yylex (void)
 {
@@ -2349,10 +2396,10 @@ yylex (void)
 	}
 
 	if (c == ':' && state->convs->range_sep_colon)
-		return RANGE_SEP;
+		return eat_space (state, RANGE_SEP);
 
 	if (c == '!' && state->convs->sheet_sep_exclamation)
-		return SHEET_SEP;
+		return eat_space (state, SHEET_SEP);
 
 	if (c == '.' && *state->ptr == '.' && state->convs->range_sep_dotdot) {
 		state->ptr++;
@@ -2378,7 +2425,7 @@ yylex (void)
 	}
 
 	if (c == state->separator)
-		return SEPARATOR;
+		return eat_space (state, SEPARATOR);
 
 	if (start != (end = state->convs->ref_parser (&ref, start, state->pos))) {
 		state->ptr = end;
@@ -2586,21 +2633,21 @@ yylex (void)
 	if (c == '<'){
 		if (*state->ptr == '='){
 			state->ptr++;
-			return LTE;
+			return eat_space (state, LTE);
 		}
 		if (*state->ptr == '>'){
 			state->ptr++;
-			return NE;
+			return eat_space (state, NE);
 		}
-		return c;
+		return eat_space (state, c);
 	}
 
 	if (c == '>'){
 		if (*state->ptr == '='){
 			state->ptr++;
-			return GTE;
+			return eat_space (state, GTE);
 		}
-		return c;
+		return eat_space (state, c);
 	}
 
 	return c;
