@@ -21,6 +21,8 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#undef GTK_DISABLE_DEPRECATED
+#warning "This file uses GTK_DISABLE_DEPRECATED"
 #include <gnumeric-config.h>
 #include "gnumeric.h"
 #include "preview-grid.h"
@@ -29,18 +31,18 @@
 #include "cell-draw.h"
 #include "colrow.h"
 #include "pattern.h"
-#include "portability.h"
 #include "mstyle.h"
 #include "rendered-value.h"
 #include "sheet-style.h"
 #include "style-border.h"
 #include "style-color.h"
 #include "value.h"
+#include "gnm-marshalers.h"
 
-#include <gal/util/e-util.h>
+#include <gsf/gsf-impl-utils.h>
 
 struct _PreviewGrid {
-	GnomeCanvasItem canvas_item;
+	FooCanvasItem canvas_item;
 
 	struct { /* Gc's */
 		GdkGC *fill;	/* Default background fill gc */
@@ -57,7 +59,7 @@ struct _PreviewGrid {
 };
 
 typedef struct {
-	GnomeCanvasItemClass parent_class;
+	FooCanvasItemClass parent_class;
 
 	int      (* get_row_height) (PreviewGrid *pg, int row);
 	int      (* get_col_width)  (PreviewGrid *pg, int col);
@@ -65,18 +67,18 @@ typedef struct {
 	Value *  (* get_cell_value) (PreviewGrid *pg, int row, int col);
 } PreviewGridClass;
 
-#define PREVIEW_GRID_CLASS(k) (GTK_CHECK_CLASS_CAST ((k), preview_grid_get_type (), PreviewGridClass))
+#define PREVIEW_GRID_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), preview_grid_get_type (), PreviewGridClass))
 
-static GnomeCanvasItemClass *preview_grid_parent_class;
+static FooCanvasItemClass *preview_grid_parent_class;
 
 /* The arguments we take */
 enum {
 	ARG_0,
-	
+
 	/* Options */
 	ARG_RENDER_GRIDLINES,
 	ARG_DEFAULT_ROW_HEIGHT,
-	ARG_DEFAULT_COL_WIDTH,
+	ARG_DEFAULT_COL_WIDTH
 };
 
 /* Signals */
@@ -94,46 +96,34 @@ typedef gpointer (*GtkSignal_POINTER__INT_INT) (GtkObject * object,
 						gint arg1,
 						gint arg2,
 						gpointer user_data);
-static void
-marshal_POINTER__INT_INT (GtkObject * object, GtkSignalFunc func,
-				 gpointer func_data, GtkArg * args)
-{
-	GtkSignal_POINTER__INT_INT rfunc;
-	gpointer *rval;
-	rval = GTK_RETLOC_POINTER (args[2]);
-	rfunc = (GtkSignal_POINTER__INT_INT) func;
-	*rval = (*rfunc) (object,
-			  GTK_VALUE_INT (args[0]),
-			  GTK_VALUE_INT (args[1]),
-			  func_data);
-}
-
 /*****************************************************************************/
 
 static int
 pg_get_row_height (PreviewGrid *pg, int const row)
 {
-	int height = pg->def.row_height;
-	
-	g_return_val_if_fail (pg != NULL, 0);
-	g_return_val_if_fail (row >= 0 && row < SHEET_MAX_ROWS, 0);
+	int height;
 
-	gtk_signal_emit (GTK_OBJECT (pg), pg_signals [GET_ROW_HEIGHT], row, &height);
+	g_return_val_if_fail (row >= 0 && row < SHEET_MAX_ROWS, 1);
 
-	return height;
+	g_signal_emit (G_OBJECT (pg), pg_signals [GET_ROW_HEIGHT], 0,
+		       row, &height);
+	if (height > 0)
+		return height;
+	return pg->def.row_height;
 }
 
 static int
 pg_get_col_width (PreviewGrid *pg, int const col)
 {
-	int width = pg->def.col_width;
-	
-	g_return_val_if_fail (pg != NULL, 0);
-	g_return_val_if_fail (col >= 0 && col < SHEET_MAX_COLS, 0);
+	int width;
 
-	gtk_signal_emit (GTK_OBJECT (pg), pg_signals [GET_COL_WIDTH], col, &width);
+	g_return_val_if_fail (col >= 0 && col < SHEET_MAX_COLS, 1);
 
-	return width;
+	g_signal_emit (G_OBJECT (pg), pg_signals [GET_COL_WIDTH], 0,
+		       col, &width);
+	if (width > 0)
+		return width;
+	return pg->def.col_width;
 }
 
 /**
@@ -151,7 +141,7 @@ pg_get_row_offset (PreviewGrid *pg, int const y, int *row_origin)
 	int pixel = 0;
 
 	g_return_val_if_fail (pg != NULL, 0);
-	
+
 	if (y < pixel) {
 		if (row_origin)
 			*row_origin = 1; /* there is a 1 pixel edge at the left */
@@ -187,7 +177,7 @@ pg_get_col_offset (PreviewGrid *pg, int const x, int *col_origin)
 	int pixel = 0;
 
 	g_return_val_if_fail (pg != NULL, 0);
-	
+
 	if (x < pixel) {
 		if (col_origin)
 			*col_origin = 1; /* there is a 1 pixel edge */
@@ -218,8 +208,9 @@ pg_get_style (PreviewGrid *pg, int const row, int const col)
 	g_return_val_if_fail (row >= 0 && row < SHEET_MAX_ROWS, 0);
 	g_return_val_if_fail (col >= 0 && col < SHEET_MAX_COLS, 0);
 
-	gtk_signal_emit (GTK_OBJECT (pg), pg_signals [GET_CELL_STYLE], row, col, &style, NULL);
-	
+	g_signal_emit (G_OBJECT (pg), pg_signals [GET_CELL_STYLE], 0,
+		       row, col, &style, NULL);
+
 	/* If no style was returned, use the default style */
 	if (style == NULL)
 		return pg->def.style;
@@ -228,10 +219,10 @@ pg_get_style (PreviewGrid *pg, int const row, int const col)
 }
 
 static Cell *
-pg_construct_cell (PreviewGrid *pg, int const row, int const col)
+pg_construct_cell (PangoContext *context, PreviewGrid *pg,
+		   int const row, int const col)
 {
 	Cell          *cell  = NULL;
-	RenderedValue *res   = NULL;
 	MStyle        *style = NULL;
 
 	g_return_val_if_fail (pg != NULL, 0);
@@ -241,15 +232,16 @@ pg_construct_cell (PreviewGrid *pg, int const row, int const col)
 	/*
 	 * We are going to manually replicate a cell
 	 * structure here, please remember that this is not
-	 * the equivalent of a real cell
+	 * the equivalent of a real cell.  In particular we g_new
+	 * it and it must be g_free'd.
 	 */
 	cell = g_new0 (Cell, 1);
-	
+
 	cell->row_info = g_new0 (ColRowInfo, 1);
 	cell->col_info = g_new0 (ColRowInfo, 1);
 
 	style = pg_get_style (pg, row, col);
-		
+
 	/*
 	 * Eventually the row_info->pos and col_info->pos
 	 * will go away
@@ -268,18 +260,12 @@ pg_construct_cell (PreviewGrid *pg, int const row, int const col)
 	cell->col_info->size_pixels = pg_get_col_width  (pg, col);
 
 	cell->value = NULL;
-	gtk_signal_emit (GTK_OBJECT (pg), pg_signals [GET_CELL_VALUE], row, col, &cell->value, NULL);
+	g_signal_emit (G_OBJECT (pg), pg_signals [GET_CELL_VALUE], 0,
+		       row, col, &cell->value, NULL);
 	if (cell->value == NULL)
 		cell->value = value_new_empty ();
-		
-	res = rendered_value_new (cell, style, TRUE);
-	cell->rendered_value = res;
 
-	/*
-	 * Rendered value needs to know text width and height to handle
-	 * alignment properly
-	 */
-	rendered_value_calc_size_ext (cell, style);
+	cell->rendered_value = rendered_value_new (cell, style, TRUE, context);
 
 	return cell;
 }
@@ -288,13 +274,13 @@ static void
 pg_destruct_cell (Cell *cell)
 {
 	g_return_if_fail (cell != NULL);
-	
+
 	value_release (cell->value);
 	rendered_value_destroy (cell->rendered_value);
 
 	g_free (cell->col_info);
 	g_free (cell->row_info);
-	
+
 	g_free (cell);
 }
 /*****************************************************************************/
@@ -323,14 +309,14 @@ preview_grid_destroy (GtkObject *object)
  * Realize handler
  **/
 static void
-preview_grid_realize (GnomeCanvasItem *item)
+preview_grid_realize (FooCanvasItem *item)
 {
 	GdkWindow   *window;
 	GtkStyle    *style;
 	PreviewGrid *pg;
 
-	if (GNOME_CANVAS_ITEM_CLASS (preview_grid_parent_class)->realize)
-		(*GNOME_CANVAS_ITEM_CLASS (preview_grid_parent_class)->realize)(item);
+	if (FOO_CANVAS_ITEM_CLASS (preview_grid_parent_class)->realize)
+		(*FOO_CANVAS_ITEM_CLASS (preview_grid_parent_class)->realize)(item);
 
 	pg = PREVIEW_GRID (item);
 	window = GTK_WIDGET (item->canvas)->window;
@@ -339,17 +325,18 @@ preview_grid_realize (GnomeCanvasItem *item)
 	 * This makes the redraws when the canvas scrolls flicker less.
 	 */
 	style = gtk_style_copy (GTK_WIDGET (item->canvas)->style);
-	style->bg [GTK_STATE_NORMAL] = gs_white;
+	style->bg[GTK_STATE_NORMAL] = style->white;
 	gtk_widget_set_style (GTK_WIDGET (item->canvas), style);
-	gtk_style_unref (style);
+	g_object_unref (style);
 
 	/* Configure the default grid gc */
 	pg->gc.fill  = gdk_gc_new (window);
 	pg->gc.cell  = gdk_gc_new (window);
 	pg->gc.empty = gdk_gc_new (window);
 
-	gdk_gc_set_foreground (pg->gc.fill, &gs_white);
-	gdk_gc_set_background (pg->gc.fill, &gs_light_gray);
+	gdk_gc_set_rgb_fg_color (pg->gc.fill, &gs_white);
+	gdk_gc_set_rgb_bg_color (pg->gc.fill, &gs_light_gray);
+	gdk_gc_set_fill (pg->gc.cell, GDK_SOLID);
 
 	/* Initialize hard-coded defaults */
 	pg->def.style = mstyle_new_default ();
@@ -362,21 +349,21 @@ preview_grid_realize (GnomeCanvasItem *item)
  * Unrealize handler
  **/
 static void
-preview_grid_unrealize (GnomeCanvasItem *item)
+preview_grid_unrealize (FooCanvasItem *item)
 {
 	PreviewGrid *pg = PREVIEW_GRID (item);
 
-	gdk_gc_unref (pg->gc.fill);
-	gdk_gc_unref (pg->gc.cell);
-	gdk_gc_unref (pg->gc.empty);
+	g_object_unref (G_OBJECT (pg->gc.fill));
+	g_object_unref (G_OBJECT (pg->gc.cell));
+	g_object_unref (G_OBJECT (pg->gc.empty));
 	pg->gc.fill  = 0;
 	pg->gc.cell  = 0;
 	pg->gc.empty = 0;
 
 	mstyle_unref (pg->def.style);
-	
-	if (GNOME_CANVAS_ITEM_CLASS (preview_grid_parent_class)->unrealize)
-		(*GNOME_CANVAS_ITEM_CLASS (preview_grid_parent_class)->unrealize)(item);
+
+	if (FOO_CANVAS_ITEM_CLASS (preview_grid_parent_class)->unrealize)
+		(*FOO_CANVAS_ITEM_CLASS (preview_grid_parent_class)->unrealize)(item);
 }
 
 /**
@@ -385,15 +372,15 @@ preview_grid_unrealize (GnomeCanvasItem *item)
  * Update handler
  **/
 static void
-preview_grid_update (GnomeCanvasItem *item, double *affine, ArtSVP *clip_path, int flags)
+preview_grid_update (FooCanvasItem *item,  double i2w_dx, double i2w_dy, int flags)
 {
-	if (GNOME_CANVAS_ITEM_CLASS (preview_grid_parent_class)->update)
-		(* GNOME_CANVAS_ITEM_CLASS (preview_grid_parent_class)->update) (item, affine, clip_path, flags);
+	if (FOO_CANVAS_ITEM_CLASS (preview_grid_parent_class)->update)
+		(* FOO_CANVAS_ITEM_CLASS (preview_grid_parent_class)->update) (item, i2w_dx, i2w_dy, flags);
 
 	item->x1 = 0;
 	item->y1 = 0;
-	item->x2 = INT_MAX;
-	item->y2 = INT_MAX;
+	item->x2 = INT_MAX/2;
+	item->y2 = INT_MAX/2;
 }
 
 /**
@@ -406,7 +393,7 @@ preview_grid_draw_background (GdkDrawable *drawable, PreviewGrid const *pg, MSty
 			      int col, int row, int x, int y, int w, int h)
 {
 	GdkGC *gc = pg->gc.empty;
-	
+
 	if (gnumeric_background_set_gc (mstyle, gc, pg->canvas_item.canvas, FALSE))
 		/* Fill the entire cell (API excludes far pixel) */
 		gdk_draw_rectangle (drawable, gc, TRUE, x, y, w+1, h+1);
@@ -464,11 +451,17 @@ pg_style_get_row (PreviewGrid *pg, StyleRow *sr)
  * The drawing routine does not support spanning.
  **/
 static void
-preview_grid_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
-		   int draw_x, int draw_y, int width, int height)
+preview_grid_draw (FooCanvasItem *item, GdkDrawable *drawable,
+		   GdkEventExpose *expose)
 {
+	gint draw_x = expose->area.x;
+	gint draw_y = expose->area.y;
+	gint width  = expose->area.width;
+	gint height = expose->area.height;
  	PreviewGrid *pg = PREVIEW_GRID (item);
- 
+	PangoContext *context = gtk_widget_get_pango_context
+		(gtk_widget_get_toplevel (GTK_WIDGET (item->canvas)));
+
  	/* To ensure that far and near borders get drawn we pretend to draw +-2
  	 * pixels around the target area which would include the surrounding
  	 * borders if necessary */
@@ -479,10 +472,10 @@ preview_grid_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
  	int x, y, col, row, n;
  	int const start_col = pg_get_col_offset (pg, draw_x - 2, &x);
  	int end_col         = pg_get_col_offset (pg, draw_x + width + 2, NULL);
- 	int const diff_x    = draw_x - x;
+ 	int const diff_x    = draw_x;
  	int start_row       = pg_get_row_offset (pg, draw_y - 2, &y);
  	int end_row         = pg_get_row_offset (pg, draw_y + height + 2, NULL);
- 	int const diff_y    = draw_y - y;
+ 	int const diff_y    = draw_y;
 
 	StyleRow sr, next_sr;
 	MStyle const **styles;
@@ -491,6 +484,8 @@ preview_grid_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
 
 	int *colwidths = NULL;
 
+	style_border_none_set_color (style_color_grid ());
+
  	/* Fill entire region with default background (even past far edge) */
  	gdk_draw_rectangle (drawable, pg->gc.fill, TRUE,
  			    draw_x, draw_y, width, height);
@@ -508,7 +503,7 @@ preview_grid_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
 	next_sr.row = sr.row = row = start_row;
 	pg_style_get_row (pg, &sr);
 
-	/* Collect the column widths */	
+	/* Collect the column widths */
 	colwidths = g_alloca (n * sizeof (int));
 	colwidths -= start_col;
 	for (col = start_col; col <= end_col; col++)
@@ -516,7 +511,7 @@ preview_grid_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
 
 	for (y = -diff_y; row <= end_row; row = sr.row = next_sr.row) {
  		int row_height = pg_get_row_height (pg, row);
- 			
+
 		if (++next_sr.row > end_row) {
 			for (col = start_col ; col <= end_col; ++col)
 				next_sr.vertical [col] =
@@ -525,15 +520,15 @@ preview_grid_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
 			pg_style_get_row (pg, &next_sr);
 
 		for (col = start_col, x = -diff_x; col <= end_col; col++) {
- 			Cell         *cell  = pg_construct_cell (pg, row, col);
+ 			Cell         *cell  = pg_construct_cell (context, pg, row, col);
 			MStyle const *style = sr.styles [col];
 
  			preview_grid_draw_background (drawable, pg,
  						      style, col, row, x, y,
  						      colwidths [col], row_height);
- 
+
 			if (!cell_is_blank (cell))
-				cell_draw (cell, style, pg->gc.cell, drawable,
+				cell_draw (cell, pg->gc.cell, drawable,
 					   x, y, -1, -1, -1);
 
 			pg_destruct_cell (cell);
@@ -550,7 +545,7 @@ preview_grid_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
 		borders = sr.top; sr.top = sr.bottom;
 		sr.bottom = next_sr.top = next_sr.bottom; next_sr.bottom = borders;
 		styles = sr.styles; sr.styles = next_sr.styles; next_sr.styles = styles;
-		 
+
 		y += row_height;
  	}
 }
@@ -563,8 +558,8 @@ preview_grid_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
  * Return value: point
  **/
 static double
-preview_grid_point (GnomeCanvasItem *item, double x, double y, int cx, int cy,
-		    GnomeCanvasItem **actual_item)
+preview_grid_point (FooCanvasItem *item, double x, double y, int cx, int cy,
+		    FooCanvasItem **actual_item)
 {
 	*actual_item = item;
 	return 0.0;
@@ -579,7 +574,7 @@ preview_grid_point (GnomeCanvasItem *item, double x, double y, int cx, int cy,
 static void
 preview_grid_init (PreviewGrid *preview_grid)
 {
-	GnomeCanvasItem *item = GNOME_CANVAS_ITEM (preview_grid);
+	FooCanvasItem *item = FOO_CANVAS_ITEM (preview_grid);
 
 	item->x1 = 0;
 	item->y1 = 0;
@@ -600,10 +595,10 @@ preview_grid_init (PreviewGrid *preview_grid)
 static void
 preview_grid_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 {
-	GnomeCanvasItem *item;
+	FooCanvasItem *item;
 	PreviewGrid *pg;
 
-	item = GNOME_CANVAS_ITEM (o);
+	item = FOO_CANVAS_ITEM (o);
 	pg = PREVIEW_GRID (o);
 
 	switch (arg_id){
@@ -631,13 +626,12 @@ preview_grid_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 static void
 preview_grid_class_init (PreviewGridClass *preview_grid_class)
 {
-	GtkObjectClass  *object_class;
-	GnomeCanvasItemClass *item_class;
+	GtkObjectClass  *klass;
+	FooCanvasItemClass *item_class;
 
-	preview_grid_parent_class = gtk_type_class (gnome_canvas_item_get_type ());
-
-	object_class = (GtkObjectClass *) preview_grid_class;
-	item_class = (GnomeCanvasItemClass *) preview_grid_class;
+	klass = (GtkObjectClass *) preview_grid_class;
+	item_class = (FooCanvasItemClass *) preview_grid_class;
+	preview_grid_parent_class = g_type_class_peek_parent (klass);
 
 	/* Manipulation */
 	gtk_object_add_arg_type ("PreviewGrid::RenderGridlines", GTK_TYPE_BOOL,
@@ -647,10 +641,10 @@ preview_grid_class_init (PreviewGridClass *preview_grid_class)
 	gtk_object_add_arg_type ("PreviewGrid::DefaultColWidth", GTK_TYPE_INT,
 				 GTK_ARG_WRITABLE, ARG_DEFAULT_COL_WIDTH);
 
-	object_class->set_arg = preview_grid_set_arg;
-	object_class->destroy = preview_grid_destroy;
+	klass->set_arg = preview_grid_set_arg;
+	klass->destroy = preview_grid_destroy;
 
-	/* GnomeCanvasItem method overrides */
+	/* FooCanvasItem method overrides */
 	item_class->update      = preview_grid_update;
 	item_class->realize     = preview_grid_realize;
 	item_class->unrealize   = preview_grid_unrealize;
@@ -658,47 +652,37 @@ preview_grid_class_init (PreviewGridClass *preview_grid_class)
 	item_class->draw        = preview_grid_draw;
 	item_class->point       = preview_grid_point;
 
-	/* Turn of translate and event handlers
-	item_class->translate   = preview_grid_translate;
-	item_class->event       = preview_grid_event;
-	*/
-
 	/* Create all the signals */
-	pg_signals [GET_ROW_HEIGHT] =
-		gtk_signal_new (
-			"get_row_height",
-			GTK_RUN_LAST,
-			object_class->type,
-			GTK_SIGNAL_OFFSET (PreviewGridClass, get_row_height),
-			gtk_marshal_INT__INT,
-			GTK_TYPE_INT, 1, GTK_TYPE_INT);
-	pg_signals [GET_COL_WIDTH] =
-		gtk_signal_new (
-			"get_col_width",
-			GTK_RUN_LAST,
-			object_class->type,
-			GTK_SIGNAL_OFFSET (PreviewGridClass, get_col_width),
-			gtk_marshal_INT__INT,
-			GTK_TYPE_INT, 1, GTK_TYPE_INT);
-	pg_signals [GET_CELL_STYLE] =
-		gtk_signal_new (
-			"get_cell_style",
-			GTK_RUN_LAST,
-			object_class->type,
-			GTK_SIGNAL_OFFSET (PreviewGridClass, get_cell_value),
-			marshal_POINTER__INT_INT,
-			GTK_TYPE_POINTER, 2, GTK_TYPE_INT, GTK_TYPE_INT);
-	pg_signals [GET_CELL_VALUE] =
-		gtk_signal_new (
-			"get_cell_value",
-			GTK_RUN_LAST,
-			object_class->type,
-			GTK_SIGNAL_OFFSET (PreviewGridClass, get_cell_value),
-			marshal_POINTER__INT_INT,
-			GTK_TYPE_POINTER, 2, GTK_TYPE_INT, GTK_TYPE_INT);
-
-	gtk_object_class_add_signals (object_class, pg_signals, LAST_SIGNAL);
+	pg_signals [GET_ROW_HEIGHT] = g_signal_new ( "get_row_height",
+		preview_grid_get_type (),
+		G_SIGNAL_RUN_LAST,
+		G_STRUCT_OFFSET (PreviewGridClass, get_row_height),
+		(GSignalAccumulator) NULL, NULL,
+		gnm__INT__INT,
+		G_TYPE_INT, 1, G_TYPE_INT);
+	pg_signals [GET_COL_WIDTH] = g_signal_new ( "get_col_width",
+		preview_grid_get_type (),
+		G_SIGNAL_RUN_LAST,
+		G_STRUCT_OFFSET (PreviewGridClass, get_col_width),
+		(GSignalAccumulator) NULL, NULL,
+		gnm__INT__INT,
+		G_TYPE_INT, 1, G_TYPE_INT);
+	pg_signals [GET_CELL_STYLE] = g_signal_new ( "get_cell_style",
+		preview_grid_get_type (),
+		G_SIGNAL_RUN_LAST,
+		G_STRUCT_OFFSET (PreviewGridClass, get_cell_value),
+		(GSignalAccumulator) NULL, NULL,
+		gnm__POINTER__INT_INT,
+		G_TYPE_POINTER, 2, G_TYPE_INT, G_TYPE_INT);
+	pg_signals [GET_CELL_VALUE] = g_signal_new ( "get_cell_value",
+		preview_grid_get_type (),
+		G_SIGNAL_RUN_LAST,
+		G_STRUCT_OFFSET (PreviewGridClass, get_cell_value),
+		(GSignalAccumulator) NULL, NULL,
+		gnm__POINTER__INT_INT,
+		G_TYPE_POINTER, 2, G_TYPE_INT, G_TYPE_INT);
 }
 
-E_MAKE_TYPE (preview_grid, "PreviewGrid", PreviewGrid,
-	     preview_grid_class_init, preview_grid_init, GNOME_TYPE_CANVAS_ITEM);
+GSF_CLASS (PreviewGrid, preview_grid,
+	   preview_grid_class_init, preview_grid_init,
+	   FOO_TYPE_CANVAS_ITEM);
