@@ -66,18 +66,17 @@
  * returns : a buffer containing the file contents
  **/
 static char *
-stf_open_and_read (GsfInput *input)
+stf_open_and_read (GsfInput *input, size_t *readsize)
 {
 	gpointer result;
 	gulong    allocsize;
-	size_t    readsize;  
 	gsf_off_t size = gsf_input_size (input);
 
 	if (gsf_input_seek (input, 0, G_SEEK_SET))
 		return NULL;
 
-	readsize = (size_t) size;
-	if ((gsf_off_t) readsize != size) /* Check for overflow */
+	*readsize = (size_t) size;
+	if ((gsf_off_t) *readsize != size) /* Check for overflow */
 		return NULL;
 	size++;
 	allocsize = (gulong) size;
@@ -87,22 +86,24 @@ stf_open_and_read (GsfInput *input)
 	if (result == NULL)
 		return NULL;
 
-	if (gsf_input_read (input, readsize, result) == NULL) {
+	*((char *)result + *readsize) = '\0';
+
+	if (size > 0 && gsf_input_read (input, *readsize, result) == NULL) {
+		g_warning ("gsf_input_read failed.");
 		g_free (result);
 		result = NULL;
 	}
 
-	*((char *)result + readsize) = '\0';
-
 	return result;
 }
 
 static char *
-stf_preparse (CommandContext *context, GsfInput *input)
+stf_preparse (CommandContext *context, GsfInput *input, gchar const *enc)
 {
-	char *data = stf_open_and_read (input);
-	unsigned char const *c;
-	int len;
+	char *data;
+	size_t len;
+
+	data = stf_open_and_read (input, &len);
 
 	if (!data) {
 		if (context)
@@ -111,42 +112,6 @@ stf_preparse (CommandContext *context, GsfInput *input)
 		return NULL;
 	}
 
-	len = stf_parse_convert_to_unix (data);
-	if (len < 0) {
-		g_free (data);
-		if (context)
-			gnumeric_error_read (context,
-				_("Error while trying to pre-convert file"));
-		return NULL;
-	}
-
-	if ((c = stf_parse_is_valid_data (data, len)) != NULL) {
-		if (context) {
-			char *msg;
-			char *invalid_char = g_locale_to_utf8 (c, 1, NULL, NULL, NULL);
-
-			/* if locale conversion failed try 8859-1 as a fallback
-			 * its ok to do this for an error message */
-			if (invalid_char == NULL)
-				invalid_char = g_convert (c, 1, "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
-			if (invalid_char != NULL) {
-				msg = g_strdup_printf (_("This file does not seem to be a valid text file.\n"
-							 "The character '%s' (ASCII decimal %d) was encountered.\n"
-							 "Most likely your locale settings are wrong."),
-						       invalid_char, (int)*c);
-				g_free (invalid_char);
-			} else
-				msg = g_strdup_printf (_("This file does not seem to be a valid text file.\n"
-							 "Byte 0x%d was encountered.\n"
-							 "Most likely your locale settings are wrong."),
-						       (int)*c);
-			gnumeric_error_read (context, msg);
-			g_free (msg);
-		}
-		g_free (data);
-		return NULL;
-	}
-
 	return data;
 }
 
@@ -154,40 +119,34 @@ static gboolean
 stf_store_results (DialogStfResult_t *dialogresult,
 		   Sheet *sheet, int start_col, int start_row)
 {
-	GSList *iterator;
-	int col, rowcount;
+	unsigned int ui;
 
 	stf_parse_options_set_lines_to_parse (dialogresult->parseoptions, dialogresult->lines);
 
-	iterator = dialogresult->formats;
-	col = start_col;
-	rowcount = stf_parse_get_rowcount (dialogresult->parseoptions, dialogresult->newstart);
-	while (iterator) {
+	for (ui = 0; ui < dialogresult->formats->len; ui++) {
+		StyleFormat *sf = g_ptr_array_index (dialogresult->formats, ui);
 		Range range;
 		MStyle *style = mstyle_new ();
 
-		mstyle_set_format (style, iterator->data);
+		mstyle_set_format (style, sf);
 
-		range.start.col = col;
+		range.start.col = start_col + ui;
 		range.start.row = start_row;
-		range.end.col   = col;
-		range.end.row   = rowcount;
+		range.end.col   = start_col + ui;
+		range.end.row   = start_row + dialogresult->rowcount - 1;
 
 		sheet_style_apply_range (sheet, &range, style);
-
-		iterator = g_slist_next (iterator);
-
-		col++;
 	}
 
 	return stf_parse_sheet (dialogresult->parseoptions,
-				dialogresult->newstart, sheet,
+				dialogresult->text, sheet,
 				start_col, start_row);
 }
 
 /**
  * stf_read_workbook
  * @fo       : file opener
+ * @enc      : encoding of file
  * @context  : command context
  * @book     : workbook
  * @input    : file to read from+convert
@@ -195,7 +154,8 @@ stf_store_results (DialogStfResult_t *dialogresult,
  * Main routine, handles importing a file including all dialog mumbo-jumbo
  **/
 static void
-stf_read_workbook (GnmFileOpener const *fo, IOContext *context, WorkbookView *wbv, GsfInput *input)
+stf_read_workbook (GnmFileOpener const *fo,  gchar const *enc,
+		   IOContext *context, WorkbookView *wbv, GsfInput *input)
 {
 	DialogStfResult_t *dialogresult = NULL;
 	char *name;
@@ -203,7 +163,7 @@ stf_read_workbook (GnmFileOpener const *fo, IOContext *context, WorkbookView *wb
 	Sheet *sheet;
 	Workbook *book;
 
-	data = stf_preparse (COMMAND_CONTEXT (context), input);
+	data = stf_preparse (COMMAND_CONTEXT (context), input, enc);
 	if (!data)
 		return;
 
@@ -216,14 +176,22 @@ stf_read_workbook (GnmFileOpener const *fo, IOContext *context, WorkbookView *wb
 	book = wb_view_workbook (wbv);
 	sheet = sheet_new (book, name);
 	workbook_sheet_attach (book, sheet, NULL);
-	g_free (name);
 
-	dialogresult = stf_dialog (WORKBOOK_CONTROL_GUI (context->impl), name, data);
+	dialogresult = stf_dialog (WORKBOOK_CONTROL_GUI (context->impl), enc, name, data);
+	g_free (name);
 	if (dialogresult != NULL && stf_store_results (dialogresult, sheet, 0, 0)) {
 		workbook_recalc (book);
 		sheet_queue_respan (sheet, 0, SHEET_MAX_ROWS-1);
-	} else
+	} else {
+		/*
+		 * FIXME: we have to set an error somehow so the caller notices
+		 * that we did nothing.  This makes it pop up an error message
+		 * which isn't quite right.
+		 */
+		gnumeric_error_read (COMMAND_CONTEXT (context),
+			_("Cancelled"));
 		workbook_sheet_detach (book, sheet);
+	}
 
 	g_free (data);
 	if (dialogresult != NULL)
@@ -241,7 +209,8 @@ cb_get_content (Sheet *sheet, int col, int row,
 			parse_pos_init_cell (&pp, cell),
 			gnm_expr_conventions_default);
 	} else if (VALUE_FMT (cell->value) != NULL)
-		tmp = format_value (NULL, cell->value, NULL, -1);
+		tmp = format_value (NULL, cell->value, NULL, -1,
+				    workbook_date_conv (sheet->workbook));
 	else
 		tmp = value_get_as_string (cell->value);
 
@@ -266,7 +235,7 @@ stf_text_to_columns (WorkbookControl *wbc, CommandContext *cc)
 	Sheet		*src_sheet, *target_sheet;
 	Range const	*src;
 	Range		 target;
-	GsfOutput	*buf;
+	GsfOutputMemory	*buf;
 	guint8 const	*data;
 
 	sv    = wb_control_cur_sheet_view (wbc);
@@ -297,11 +266,12 @@ stf_text_to_columns (WorkbookControl *wbc, CommandContext *cc)
 		src->end.col, src->end.row,
 		(CellIterFunc) &cb_get_content, buf);
 
-	gsf_output_write (buf, 1, "\0");
-	gsf_output_close (buf);
-	data = gsf_output_memory_get_bytes (GSF_OUTPUT_MEMORY (buf));
+	gsf_output_write (GSF_OUTPUT (buf), 1, "\0");
+	gsf_output_close (GSF_OUTPUT (buf));
+	data = gsf_output_memory_get_bytes (buf);
 	dialogresult = stf_dialog (WORKBOOK_CONTROL_GUI (wbc),
-		_("Text to Columns"), data);
+				   NULL,
+				   _("Text to Columns"), data);
 
 	if (dialogresult == NULL ||
 	    !stf_store_results (dialogresult, target_sheet,
@@ -325,6 +295,7 @@ stf_text_to_columns (WorkbookControl *wbc, CommandContext *cc)
 /**
  * stf_read_workbook_auto_csvtab
  * @fo       : file opener
+ * @enc      : optional encoding
  * @context  : command context
  * @book     : workbook
  * @input    : file to read from+convert
@@ -332,7 +303,8 @@ stf_text_to_columns (WorkbookControl *wbc, CommandContext *cc)
  * Attempt to auto-detect CSV or tab-delimited file
  **/
 static void
-stf_read_workbook_auto_csvtab (GnmFileOpener const *fo, IOContext *context,
+stf_read_workbook_auto_csvtab (GnmFileOpener const *fo, gchar const *enc,
+			       IOContext *context,
 			       WorkbookView *wbv, GsfInput *input)
 {
 	Sheet *sheet;
@@ -340,41 +312,56 @@ stf_read_workbook_auto_csvtab (GnmFileOpener const *fo, IOContext *context,
 	char *name;
 	char *data;
 	StfParseOptions_t *po;
-	char *pos;
-	unsigned int comma = 0, tab = 0, lines = 0;
+	char const *pos;
+	unsigned int sep = 0, tab = 0, lines = 0;
 	int i;
+	gboolean last_was_newline = FALSE;
+
+        gunichar guni_tab = '\t';
+	gunichar guni_newline = '\n';
+	gunichar guni_carriage = '\r';
+	gunichar guni_sep = format_get_arg_sep ();
 
 	book = wb_view_workbook (wbv);
-	data = stf_preparse (COMMAND_CONTEXT (context), input);
+	data = stf_preparse (COMMAND_CONTEXT (context), input, enc);
 	if (!data)
 		return;
 
-        for (i = STF_PROBE_SIZE, pos = data ; *pos && i-- > 0; ++pos )
-		if (*pos == ',')
-			++comma;
-		else if (*pos == '\t')
+        po = stf_parse_options_new ();
+
+	stf_parse_options_set_type (po, PARSE_TYPE_CSV);
+	stf_parse_options_set_trim_spaces (po, TRIM_TYPE_LEFT | TRIM_TYPE_RIGHT);
+        stf_parse_options_set_lines_to_parse (po, -1);
+	stf_parse_options_csv_set_stringindicator (po, '"');
+	stf_parse_options_csv_set_indicator_2x_is_single (po, TRUE);
+	stf_parse_options_csv_set_duplicates (po, FALSE);
+
+        for (i = STF_PROBE_SIZE, pos = data ; pos && *pos && i-- > 0;
+	     pos = stf_parse_next_token (pos, po, NULL)) {
+		gunichar this_char;
+
+		this_char = g_utf8_get_char (pos);
+		if (this_char == guni_sep) {
+			++sep;
+			last_was_newline = FALSE;
+		} else if (this_char == guni_tab) {
 			++tab;
-		else if (*pos == '\n')
+			last_was_newline = FALSE;
+		} else if ((this_char == guni_newline || this_char == guni_carriage)
+			   && !last_was_newline) {
 			++lines;
+			last_was_newline = TRUE;
+		}
+	}
 
 	name = g_path_get_basename (gsf_input_name (input));
 	sheet = sheet_new (book, name);
 
 	workbook_sheet_attach (book, sheet, NULL);
 
-	po = stf_parse_options_new ();
-
-	stf_parse_options_set_type (po, PARSE_TYPE_CSV);
-	stf_parse_options_set_trim_spaces (po, TRIM_TYPE_LEFT | TRIM_TYPE_RIGHT);
-	stf_parse_options_set_lines_to_parse (po, -1);
-
-	stf_parse_options_csv_set_stringindicator (po, '"');
-	stf_parse_options_csv_set_indicator_2x_is_single (po, FALSE);
-	stf_parse_options_csv_set_duplicates (po, FALSE);
-
 	/* Guess */
-	stf_parse_options_csv_set_separators (po, ",", NULL);
-	if (tab >= lines && tab > comma)
+	stf_parse_options_csv_set_separators (po, (guni_sep == ',') ? ",":";", NULL);
+	if (tab > lines || tab >= sep)
 		stf_parse_options_csv_set_separators (po, "\t", NULL);
 
 	if (!stf_parse_sheet (po, data, sheet, 0, 0)) {
@@ -395,24 +382,6 @@ stf_read_workbook_auto_csvtab (GnmFileOpener const *fo, IOContext *context,
 	g_free (data);
 }
 
-static gboolean
-stf_read_default_probe (GnmFileOpener const *fo, GsfInput *input, FileProbeLevel pl)
-{
-	guint8 const *data;
-	gsf_off_t remain;
-	int len;
-
-	if (pl < FILE_PROBE_CONTENT_FULL)
-		return FALSE;
-
-	if (gsf_input_seek (input, 0, G_SEEK_SET))
-		return FALSE;
-	remain = gsf_input_remaining (input);
-	len = MIN (remain, STF_PROBE_SIZE);
-	data = gsf_input_read (input, len, NULL);
-	return (data != NULL) && (stf_parse_is_valid_data (data, len) == NULL);
-}
-
 /***********************************************************************************/
 
 static gboolean
@@ -462,15 +431,15 @@ stf_write_workbook (GnmFileSaver const *fs, IOContext *context,
 void
 stf_init (void)
 {
-	register_file_opener (gnm_file_opener_new (
+	gnm_file_opener_register (gnm_file_opener_new_with_enc (
 		"Gnumeric_stf:stf_csvtab",
-		_("Comma or tab separated files (CSV/TSV))"),
-		stf_read_default_probe, stf_read_workbook_auto_csvtab), 0);
-	register_file_opener (gnm_file_opener_new (
+		_("Comma or tab separated files (CSV/TSV)"),
+		NULL, stf_read_workbook_auto_csvtab), 0);
+	gnm_file_opener_register (gnm_file_opener_new_with_enc (
 		"Gnumeric_stf:stf_druid",
 		_("Text import (configurable)"),
 		NULL, stf_read_workbook), 0);
-	register_file_saver (gnm_file_saver_new (
+	gnm_file_saver_register (gnm_file_saver_new (
 		"Gnumeric_stf:stf", "csv",
 		_("Text export (configurable)"),
 		FILE_FL_WRITE_ONLY, stf_write_workbook));
